from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#


import datetime
from . import fixture

import mock

from subscription_manager.ga import GObject as ga_GObject
import subscription_manager.injection as inj
from subscription_manager.injection import provide
from subscription_manager import async_utils
from subscription_manager import managerlib

from . import stubs


# some bits we end up calling from list pools
class ListPoolsStubUEP(stubs.StubUEP):
    def getOwner(self, consumeruuid):
        return {'key': 'owner'}

    def getPoolsList(self, consumer, listAll=None, active_on=None, owner=None):
        return []

    def getEntitlementList(self, consumeruuid=None):
        return []


class TestAsyncPool(fixture.SubManFixture):
    def setUp(self):
        self.callbacks = []
        super(TestAsyncPool, self).setUp()

    def thread_queue_callback(self, data, error):
        self.callbacks.append((data, error))

    def idle_callback(self, *args):
        # hit the refresh a few times, out stubbed
        # refresh doesn't really do anything though
        self.ap.refresh(datetime.date.today(), self.thread_queue_callback)
        if len(self.callbacks) > 3:
            self.mainloop.quit()
        return True

    def _create_async_pool(self):
        provide(inj.CP_PROVIDER, stubs.StubCPProvider())
        inj.provide(inj.PROD_DIR, stubs.StubProductDirectory())
        inj.provide(inj.ENT_DIR, stubs.StubEntitlementDirectory())
        inj.provide(inj.CERT_SORTER, stubs.StubCertSorter())

        self.pool_stash = \
                managerlib.PoolStash()

        self.ap = async_utils.AsyncPool(self.pool_stash)

        # add a timeout and a idle handler
        self.idle = ga_GObject.idle_add(self.ap.refresh, datetime.date.today(), self.idle_callback)
        self.timer = ga_GObject.timeout_add(50, self.idle_callback)
        self.mainloop = ga_GObject.MainLoop()

    def test(self):
        self._create_async_pool()

        self.mainloop.run()
        # verify our callback got called a few times
        self.assertTrue(len(self.callbacks) > 3)

    def test_exception(self):
        self._create_async_pool()

        # simulate a exception on pool refresh
        self.pool_stash.refresh = mock.Mock()
        self.pool_stash.refresh.side_effect = IOError()

        self.mainloop.run()
        self.assertTrue(len(self.callbacks) > 3)
        # we should have an sys.exc_info tuple in the error from the callback
        self.assertTrue(isinstance(self.callbacks[0][1], tuple))
        self.assertTrue(isinstance(self.callbacks[0][1][1], IOError))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
Module for testing Python all modules from Python package rhsmlib.cloud
"""

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from mock import patch, Mock
import tempfile
import time
import base64

from rhsmlib.cloud.providers import aws, azure, gcp
from rhsmlib.cloud.utils import detect_cloud_provider, collect_cloud_info


class TestAWSDetector(unittest.TestCase):
    """
    Class used for testing detector of AWS
    """

    def test_aws_not_vm(self):
        """
        Test for the case, when the machine is host (not virtual machine)
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': False,
            'dmi.bios.version': 'cool hardware company'
        }
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertFalse(is_vm)

    def test_aws_vm_using_xen(self):
        """
        Test for the case, when the vm is running on AWS Xen
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'xen',
            'dmi.bios.version': 'amazon'
        }
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_xen_vm = aws_detector.is_running_on_cloud()
        self.assertTrue(is_aws_xen_vm)

    def test_aws_vm_using_kvm(self):
        """
        Test for the case, when the vm is running on AWS KVM
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm',
            'dmi.bios.version': '1.0',
            'dmi.bios.vendor': 'Amazon EC2'
        }
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_kvm_vm = aws_detector.is_running_on_cloud()
        self.assertTrue(is_aws_kvm_vm)

    def test_vm_not_on_aws_cloud(self):
        """
        Test for the case, when the vm is not running on AWS
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm',
            'dmi.bios.version': '1.0',
            'dmi.bios.vendor': 'Foo'
        }
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_vm = aws_detector.is_running_on_cloud()
        self.assertFalse(is_aws_vm)

    def test_vm_without_dmi_bios_info(self):
        """
        Test for the case, when SM BIOS does not provide any useful information for our code
        """
        # We will mock facts using simple dictionary
        facts = {}
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertFalse(is_vm)
        is_aws_vm = aws_detector.is_running_on_cloud()
        self.assertFalse(is_aws_vm)

    def test_vm_system_uuid_starts_with_ec2(self):
        """
        Test for the case, when system UUID starts with EC2 string as it is described here:
        https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/identify_ec2_instances.html
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'dmi.system.uuid': 'EC2263F8-15F3-4A34-B186-FAD8AB963431'
        }
        aws_detector = aws.AWSCloudDetector(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        probability = aws_detector.is_likely_running_on_cloud()
        self.assertEqual(probability, 0.1)


class TestAzureDetector(unittest.TestCase):
    """
    Class used for testing detector of Azure
    """

    def test_azure_not_vm(self):
        """
        Test for the case, when the machine is host (not virtual machine)
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': False,
            'dmi.bios.version': 'cool hardware company'
        }
        azure_detector = azure.AzureCloudDetector(facts)
        is_vm = azure_detector.is_vm()
        self.assertFalse(is_vm)

    def test_azure_vm(self):
        """
        Test for the case, when the vm is running on Azure
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'hyperv',
            'dmi.bios.version': '090008',
            'dmi.chassis.asset_tag': '7783-7084-3265-9085-8269-3286-77'
        }
        azure_detector = azure.AzureCloudDetector(facts)
        is_vm = azure_detector.is_vm()
        self.assertTrue(is_vm)
        is_azure_vm = azure_detector.is_running_on_cloud()
        self.assertTrue(is_azure_vm)

    def test_vm_not_on_azure_cloud(self):
        """
        Test for the case, when the vm is not running on AWS
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'hyperv',
            'dmi.bios.version': '090008',
            'dmi.bios.vendor': 'Foo'
        }
        azure_detector = azure.AzureCloudDetector(facts)
        is_vm = azure_detector.is_vm()
        self.assertTrue(is_vm)
        is_azure_vm = azure_detector.is_running_on_cloud()
        self.assertFalse(is_azure_vm)

    def test_vm_without_dmi_bios_info(self):
        """
        Test for the case, when MS BIOS does not provide any useful information for our code
        """
        # We will mock facts using simple dictionary
        facts = {}
        azure_detector = azure.AzureCloudDetector(facts)
        is_vm = azure_detector.is_vm()
        self.assertFalse(is_vm)
        is_azure_vm = azure_detector.is_running_on_cloud()
        self.assertFalse(is_azure_vm)


class TestGCPDetector(unittest.TestCase):
    """
    Class used for testing detector of GCP
    """

    def test_gcp_not_vm(self):
        """
        Test for the case, when the machine is host (not virtual machine)
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': False,
            'dmi.bios.version': 'cool hardware company'
        }
        gcp_detector = gcp.GCPCloudDetector(facts)
        is_vm = gcp_detector.is_vm()
        self.assertFalse(is_vm)

    def test_gcp_vm(self):
        """
        Test for the case, when the vm is running on GCP
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm',
            'dmi.bios.version': 'Google',
            'dmi.bios.vendor': 'Google'
        }
        gcp_detector = gcp.GCPCloudDetector(facts)
        is_vm = gcp_detector.is_vm()
        self.assertTrue(is_vm)
        is_gcp_vm = gcp_detector.is_running_on_cloud()
        self.assertTrue(is_gcp_vm)

    def test_vm_not_on_gcp_cloud(self):
        """
        Test for the case, when the vm is not running on GCP
        """
        # We will mock facts using simple dictionary
        facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm',
            'dmi.bios.version': '1.0',
            'dmi.bios.vendor': 'Foo'
        }
        gcp_detector = gcp.GCPCloudDetector(facts)
        is_vm = gcp_detector.is_vm()
        self.assertTrue(is_vm)
        is_gcp_vm = gcp_detector.is_running_on_cloud()
        self.assertFalse(is_gcp_vm)


def get_only_imds_v2_is_supported(url, headers, *args, **kwargs):
    """
    Mock result, when we try to get metadata using GET method against
    AWS metadata provider. This mock is for the case, when only IMDSv2
    is supported by instance.
    :param url: URL
    :param headers: HTTP headers
    :param args: other position argument
    :param kwargs: other keyed argument
    :return: Mock with result
    """
    if 'X-aws-ec2-metadata-token' in headers.keys():
        if headers['X-aws-ec2-metadata-token'] == AWS_TOKEN:
            if url == aws.AWSCloudCollector.CLOUD_PROVIDER_METADATA_URL:
                mock_result = Mock()
                mock_result.status_code = 200
                mock_result.text = AWS_METADATA
            elif url == aws.AWSCloudCollector.CLOUD_PROVIDER_SIGNATURE_URL:
                mock_result = Mock()
                mock_result.status_code = 200
                mock_result.text = AWS_SIGNATURE
            else:
                mock_result = Mock()
                mock_result.status_code = 400
                mock_result.text = 'Error: Invalid URL'
        else:
            mock_result = Mock()
            mock_result.status_code = 400
            mock_result.text = 'Error: Invalid metadata token provided'
    else:
        mock_result = Mock()
        mock_result.status_code = 400
        mock_result.text = 'Error: IMDSv1 is not supported on this instance'
    return mock_result


def put_imds_v2_token(url, headers, *args, **kwargs):
    """
    Mock getting metadata token using PUT method against AWS metadata provider
    :param url: URL
    :param headers: HTTP header
    :param args: other position arguments
    :param kwargs: other keyed arguments
    :return: Mock with response
    """
    if url == aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_URL:
        if 'X-aws-ec2-metadata-token-ttl-seconds' in headers:
            mock_result = Mock()
            mock_result.status_code = 200
            mock_result.text = AWS_TOKEN
        else:
            mock_result = Mock()
            mock_result.status_code = 400
            mock_result.text = 'Error: TTL for token not specified'
    else:
        mock_result = Mock()
        mock_result.status_code = 400
        mock_result.text = 'Error: Invalid URL'
    return mock_result


AWS_METADATA = """
{
  "accountId" : "012345678900",
  "architecture" : "x86_64",
  "availabilityZone" : "eu-central-1b",
  "billingProducts" : [ "bp-0124abcd" ],
  "devpayProductCodes" : null,
  "marketplaceProductCodes" : null,
  "imageId" : "ami-0123456789abcdeff",
  "instanceId" : "i-abcdef01234567890",
  "instanceType" : "m5.large",
  "kernelId" : null,
  "pendingTime" : "2020-02-02T02:02:02Z",
  "privateIp" : "12.34.56.78",
  "ramdiskId" : null,
  "region" : "eu-central-1",
  "version" : "2017-09-30"
}
"""

AWS_SIGNATURE = """ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz01234567899w0BBwGggCSABIIB
73sKICAiYWNjb3VudElkIiA6ICI1NjcwMTQ3ODY4OTAiLAogICJhcmNoaXRlY3R1cmUiIDogIng4
Nl82NCIsCiAgImF2YWlsYWJpbGl0eVpvbmUiIDogImV1LWNlbnRyYWwtMWIiLAogICJiaWxsaW5n
UHJvZHVjdHMiIDogWyAiYnAtNmZhNTQwMDYiIF0sCiAgImRldnBheVByb2R1Y3RDb2RlcyIgOiBu
73sKICAiYWNjb3VudElkIiA6ICI1NjcwMTQ3ODY4OTAiLAogICJhcmNoaXRlY3R1cmUiIDogIng4
bWktMDgxNmFkNzYyOTc2YzM1ZGIiLAogICJpbnN0YW5jZUlkIiA6ICJpLTBkNTU0YzRmM2JhNWVl
YTczIiwKICAiaW5zdGFuY2VUeXBlIiA6ICJtNS5sYXJnZSIsCiAgImtlcm5lbElkIiA6IG51bGws
CiAgInBlbmRpbmdUaW1lIiA6ICIyMDIwLTA0LTI0VDE0OjU3OjQzWiIsCiAgInByaXZhdGVJcCIg
OiAiMTcyLjMxLjExLjc4IiwKICAicmFtZGlza0lkIiA6IG51bGwsCiAgInJlZ2lvbiIgOiAiZXUt
Y2VudHJhbC0xIiwKICAidmVyc2lvbiIgOiAiMjAxNy0wOS0zMCIKfQAAAAAAADGCAf8wggH7AgEB
CiAgInBlbmRpbmdUaW1lIiA6ICIyMDIwLTA0LTI0VDE0OjU3OjQzWiIsCiAgInByaXZhdGVJcCIg
YXR0bGUxIDAeBgNVBAoTF0FtYXpvbiBXZWIgU2VydmljZXMgTExDAgkAoP6/ot5H9aswDQYJYIZI
AWUDBAIBBQCgaTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMDA5
MDExNTMyNDhaMC8GCSqGSIb3DQEJBDEiBCCmizT0hDlJmxHtDBaEjql5ZPFaoKy6OSk7qBFREVRk
iTANBgkqhkiG9w0BAQEFAASCAQAh//5+AaFAcgw/5SoglQ27kQKuThcJYa+QhC2aw4n1GvkvCmyi
helVMxH33tB9tUei/mapSF3v8jUseRLEbcDVRHf6n6h14Qj2MxtgYanzUCDF8qECYbZ2uSy3JLEP
iNsndm8nt7XcJC7NRoWJWAsly1VeXVIauA/l7uXmUarDQs5BhFYl7REX4htxg9mCibR6xqU5i8/D
iTANBgkqhkiG9w0BAQEFAASCAQAh//5+AaFAcgw/5SoglQ27kQKuThcJYa+QhC2aw4n1GvkvCmyi
tGbafapTj+6KnJAfP0sW7ZbzKclaCPHXQ37z9mc8vtCxEQmCbGL6sj2wtpi4rmRlAAAAAAAA"""

AWS_TOKEN = "ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz0123=="


class TestAWSCollector(unittest.TestCase):
    """
    Test case for AWSCloudCollector
    """

    def setUp(self):
        """
        Patch communication with metadata provider
        """
        requests_patcher = patch('rhsmlib.cloud.providers.aws.requests')
        self.requests_mock = requests_patcher.start()
        self.addCleanup(requests_patcher.stop)

    def test_get_metadata_from_server_imds_v1(self):
        """
        Test the case, when metadata are obtained from server using IMDSv1
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AWS_METADATA
        self.requests_mock.get = Mock(return_value=mock_result)
        aws_collector = aws.AWSCloudCollector()
        # Mock that no metadata cache exists
        aws_collector._get_metadata_from_cache = Mock(return_value=None)
        metadata = aws_collector.get_metadata()
        self.assertEqual(metadata, AWS_METADATA)

    def test_get_signature_from_server_imds_v1(self):
        """
        Test the case, when metadata are obtained from server using IMDSv1
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AWS_SIGNATURE
        self.requests_mock.get = Mock(return_value=mock_result)
        aws_collector = aws.AWSCloudCollector()
        # Mock that no metadata cache exists
        aws_collector._get_signature_from_cache = Mock(return_value=None)
        test_signature = aws_collector.get_signature()
        signature = '-----BEGIN PKCS7-----\n' + AWS_SIGNATURE + '\n-----END PKCS7-----'
        self.assertEqual(signature, test_signature)

    def test_get_metadata_from_server_imds2(self):
        """
        Test the case, when metadata are obtained from server using IMDSv2
        """
        self.requests_mock.get = get_only_imds_v2_is_supported
        self.requests_mock.put = put_imds_v2_token

        aws_collector = aws.AWSCloudCollector()
        # Mock that no metadata cache exists
        aws_collector._get_metadata_from_cache = Mock(return_value=None)
        # Mock that no token cache exists
        aws_collector._get_token_from_cache_file = Mock(return_value=None)
        # Mock writing token to cache file
        aws_collector._write_token_to_cache_file = Mock()

        metadata = aws_collector.get_metadata()
        self.assertEqual(metadata, AWS_METADATA)

    def test_get_signature_from_server_imds2(self):
        """
        Test the case, when signature is obtained from server using IMDSv2
        """
        self.requests_mock.get = get_only_imds_v2_is_supported
        self.requests_mock.put = put_imds_v2_token

        aws_collector = aws.AWSCloudCollector()
        # Mock that no metadata cache exists
        aws_collector._get_metadata_from_cache = Mock(return_value=None)
        # Mock that no token cache exists
        aws_collector._get_token_from_cache_file = Mock(return_value=None)
        # Mock writing token to cache file
        aws_collector._write_token_to_cache_file = Mock()

        test_signature = aws_collector.get_signature()
        signature = '-----BEGIN PKCS7-----\n' + AWS_SIGNATURE + '\n-----END PKCS7-----'
        self.assertEqual(signature, test_signature)

    def test_reading_valid_cached_token(self):
        """
        Test reading of valid cached token from file
        """
        c_time = str(time.time())
        ttl = str(aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_TTL)
        token = 'ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=='
        valid_token = '{\
  "ctime": "' + c_time + '",\
  "ttl": "' + ttl + '",\
  "token": "' + token + '"\
}'
        aws_collector = aws.AWSCloudCollector()
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding='utf-8'))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(token, test_token)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_corrupted_json_file(self):
        """
        Test reading of corrupted cached token from file
        """
        c_time = str(time.time())
        ttl = str(aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_TTL)
        token = 'ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=='
        valid_token = '{[\
  "ctime": "' + c_time + '",\
  "ttl": "' + ttl + '",\
  "token": "' + token + '"\
}]'
        aws_collector = aws.AWSCloudCollector()
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding='utf-8'))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_wrong_ctime_format(self):
        """
        Test reading of cached token from file with invalid time format
        """
        ttl = str(aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_TTL)
        token = 'ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=='
        # ctime has to be float (unix epoch)
        valid_token = '{[\
  "ctime": "Wed Dec 16 16:31:36 CET 2020",\
  "ttl": "' + ttl + '",\
  "token": "' + token + '"\
}]'
        aws_collector = aws.AWSCloudCollector()
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding='utf-8'))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_missing_key(self):
        """
        Test reading of invalid cached token from file
        """
        valid_token = '{\
  "token": "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="\
}'
        aws_collector = aws.AWSCloudCollector()
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding='utf-8'))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_timed_out_cached_token(self):
        """
        Test reading of cached token from file that is timed out
        """
        c_time = str(time.time() - aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_TTL - 10)
        ttl = str(aws.AWSCloudCollector.CLOUD_PROVIDER_TOKEN_TTL)
        valid_token = '{\
  "ctime": "' + c_time + '",\
  "ttl": "' + ttl + '",\
  "token": "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="\
}'
        aws_collector = aws.AWSCloudCollector()
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding='utf-8'))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file


AZURE_METADATA = """
{
    "compute": {
        "azEnvironment": "AzurePublicCloud",
        "customData": "",
        "location": "westeurope",
        "name": "foo-bar",
        "offer": "RHEL",
        "osType": "Linux",
        "placementGroupId": "",
        "plan": {
            "name": "",
            "product": "",
            "publisher": ""
        },
        "platformFaultDomain": "0",
        "platformUpdateDomain": "0",
        "provider": "Microsoft.Compute",
        "publicKeys": [
            {
                "keyData": "ssh-rsa SOMEpublicSSHkey user@localhost.localdomain",
                "path": "/home/user/.ssh/authorized_keys"
            }
        ],
        "publisher": "RedHat",
        "resourceGroupName": "foo-bar",
        "resourceId": "/subscriptions/01234567-0123-0123-0123-012345679abc/resourceGroups/foo-bar/providers/Microsoft.Compute/virtualMachines/foo",
        "sku": "8.1-ci",
        "storageProfile": {
            "dataDisks": [],
            "imageReference": {
                "id": "",
                "offer": "RHEL",
                "publisher": "RedHat",
                "sku": "8.1-ci",
                "version": "latest"
            },
            "osDisk": {
                "caching": "ReadWrite",
                "createOption": "FromImage",
                "diskSizeGB": "64",
                "encryptionSettings": {
                    "enabled": "false"
                },
                "image": {
                    "uri": ""
                },
                "managedDisk": {
                    "id": "/subscriptions/01234567-0123-0123-0123-012345679abc/resourceGroups/FOO-BAR/providers/Microsoft.Compute/disks/foo_OsDisk_1_b21768daf38e48c6a0db7cff1f054b03",
                    "storageAccountType": ""
                },
                "name": "FOO_OsDisk_1_b21768daf38e48c6a0db7cff1f054b03",
                "osType": "Linux",
                "vhd": {
                    "uri": ""
                },
                "writeAcceleratorEnabled": "false"
            }
        },
        "subscriptionId": "01234567-0123-0123-0123-012345679abc",
        "tags": "",
        "version": "8.1.2020042511",
        "vmId": "12345678-1234-1234-1234-123456789abc",
        "vmScaleSetName": "",
        "vmSize": "Standard_D2s_v3",
        "zone": ""
    },
    "network": {
        "interface": [
            {
                "ipv4": {
                    "ipAddress": [
                        {
                            "privateIpAddress": "172.16.2.5",
                            "publicIpAddress": "1.2.3.4"
                        }
                    ],
                    "subnet": [
                        {
                            "address": "172.16.2.0",
                            "prefix": "24"
                        }
                    ]
                },
                "ipv6": {
                    "ipAddress": []
                },
                "macAddress": "000D3A123456"
            }
        ]
    }
}
"""


AZURE_SIGNATURE = """
{"encoding":"pkcs7","signature":"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"}
"""


class TestAzureCollector(unittest.TestCase):
    """
    Class for testing Azure collector implemented in rhsmlib.cloud.providers.azure
    """

    def setUp(self):
        """
        Patch communication with metadata provider
        """
        requests_patcher = patch('rhsmlib.cloud.collector.requests')
        self.requests_mock = requests_patcher.start()
        self.addCleanup(requests_patcher.stop)

    def test_get_metadata_from_server(self):
        """
        Test getting metadata from server, when there is no cache
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AZURE_METADATA
        self.requests_mock.get = Mock(return_value=mock_result)
        azure_collector = azure.AzureCloudCollector()
        metadata = azure_collector.get_metadata()
        self.requests_mock.get.assert_called_once_with(
            "http://169.254.169.254/metadata/instance?api-version=2020-09-01",
            headers={
                'user-agent': 'RHSM/1.0',
                "Metadata": "true"
            }
        )
        self.assertEqual(metadata, AZURE_METADATA)

    def test_get_signature_from_server(self):
        """
        Test getting signature from server, when there is no cache file
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AZURE_SIGNATURE
        self.requests_mock.get = Mock(return_value=mock_result)
        azure_collector = azure.AzureCloudCollector()
        signature = azure_collector.get_signature()
        self.requests_mock.get.assert_called_once_with(
            "http://169.254.169.254/metadata/attested/document?api-version=2020-09-01",
            headers={
                'user-agent': 'RHSM/1.0',
                "Metadata": "true"
            }
        )
        self.assertEqual(signature, AZURE_SIGNATURE)


GCP_TOKEN = "VERYveryVERYlongTOKEN"


class TestGCPCollector(unittest.TestCase):
    """
    Class for testing Google Cloud Platform (GCP) collector implemented in rhsmlib.cloud.providers.gcp
    """

    def setUp(self):
        """
        Patch communication with metadata provider
        """
        requests_patcher = patch('rhsmlib.cloud.collector.requests')
        self.requests_mock = requests_patcher.start()
        self.addCleanup(requests_patcher.stop)

    def test_get_token(self):
        """
        Test getting GCP token, when default audience URL is used
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = GCP_TOKEN
        self.requests_mock.get = Mock(return_value=mock_result)
        gcp_collector = gcp.GCPCloudCollector()
        gcp_collector._get_token_from_cache_file = Mock(return_value=None)
        gcp_collector._write_token_to_cache_file = Mock(return_value=None)
        token = gcp_collector.get_metadata()
        self.requests_mock.get.assert_called_once_with(
            'http://metadata/computeMetadata/v1/instance/service-accounts/default/identity?audience=https://subscription.rhsm.redhat.com:443/subscription&format=full',
            headers={
                'user-agent': 'RHSM/1.0',
                'Metadata-Flavor': 'Google'
            }
        )
        self.assertEqual(token, GCP_TOKEN)

    def test_get_token_custom_audience(self):
        """
        Test getting GCP token, when custom audience URL is used (e.g. Satellite or stage is used)
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = GCP_TOKEN
        self.requests_mock.get = Mock(return_value=mock_result)
        gcp_collector = gcp.GCPCloudCollector(audience_url="https://example.com:8443/rhsm")
        gcp_collector._get_token_from_cache_file = Mock(return_value=None)
        gcp_collector._write_token_to_cache_file = Mock(return_value=None)
        token = gcp_collector.get_metadata()
        self.requests_mock.get.assert_called_once_with(
            'http://metadata/computeMetadata/v1/instance/service-accounts/default/identity?audience=https://example.com:8443/rhsm&format=full',
            headers={
                'user-agent': 'RHSM/1.0',
                'Metadata-Flavor': 'Google'
            }
        )
        self.assertEqual(token, GCP_TOKEN)


class TestCloudUtils(unittest.TestCase):
    """
    Class for testing rhsmlib.cloud.utils module
    """
    def setUp(self):
        """
        Set up two mocks that are used in all tests
        """
        host_collector_patcher = patch('rhsmlib.cloud.utils.HostCollector')
        self.host_collector_mock = host_collector_patcher.start()
        self.host_fact_collector_instance = Mock()
        self.host_collector_mock.return_value = self.host_fact_collector_instance
        self.addCleanup(host_collector_patcher.stop)

        hardware_collector_patcher = patch('rhsmlib.cloud.utils.HardwareCollector')
        self.hardware_collector_mock = hardware_collector_patcher.start()
        self.hw_fact_collector_instance = Mock()
        self.hardware_collector_mock.return_value = self.hw_fact_collector_instance
        self.addCleanup(hardware_collector_patcher.stop)

        aws_requests_patcher = patch('rhsmlib.cloud.providers.aws.requests')
        self.aws_requests_mock = aws_requests_patcher.start()
        self.addCleanup(aws_requests_patcher.stop)
        write_cache_patcher = patch('rhsmlib.cloud.providers.aws.AWSCloudCollector._write_token_to_cache_file')
        self.write_cache_mock = write_cache_patcher.start()
        self.addCleanup(write_cache_patcher.stop)

        azure_requests_patcher = patch('rhsmlib.cloud.collector.requests')
        self.azure_requests_mock = azure_requests_patcher.start()
        self.addCleanup(azure_requests_patcher.stop)

    def test_detect_cloud_provider_aws(self):
        """
        Test the case, when detecting of aws works as expected
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm'
        }
        hw_facts = {
            'dmi.bios.vendor': 'Amazon EC2'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['aws'])

    def test_detect_cloud_provider_aws_heuristics(self):
        """
        Test the case, when detecting of aws does not work using strong signs, but it is necessary
        to use heuristics method
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm'
        }
        hw_facts = {
            'dmi.bios.vendor': 'AWS',
            'dmi.bios.version': '1.0'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['aws', 'gcp'])

    def test_detect_cloud_provider_gcp(self):
        """
        Test the case, when detecting of gcp works as expected
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm'
        }
        hw_facts = {
            'dmi.bios.vendor': 'Google',
            'dmi.bios.version': 'Google'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['gcp'])

    def test_detect_cloud_provider_gcp_heuristics(self):
        """
        Test the case, when detecting of gcp does not work using strong signs, but it is necessary
        to use heuristics method
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm'
        }
        hw_facts = {
            'dmi.bios.vendor': 'Foo Company',
            'dmi.bios.version': '1.0',
            'dmi.chassis.asset_tag': 'Google Cloud'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['gcp', 'aws'])

    def test_detect_cloud_provider_azure(self):
        """
        Test the case, when detecting of azure works as expected
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'hyperv',
        }
        hw_facts = {
            'dmi.bios.vendor': 'Foo company',
            'dmi.bios.version': '1.0',
            'dmi.chassis.asset_tag': '7783-7084-3265-9085-8269-3286-77'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['azure'])

    def test_detect_cloud_provider_azure_heuristics(self):
        """
        Test the case, when detecting of azure does not work using strong signs, but it is necessary
        to use heuristics method
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'hyperv',
        }
        hw_facts = {
            'dmi.bios.vendor': 'Microsoft',
            'dmi.bios.version': '1.0',
            'dmi.system.manufacturer': 'Google',
            'dmi.chassis.manufacturer': 'Amazon'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['azure', 'gcp', 'aws'])

    def test_conclict_in_strong_signs(self):
        """
        Test the case, when cloud providers change strong signs and there is conflict (two providers
        are detected using strong signs). In such case result using strong signs should be dropped
        and heuristics should be used, because strong signs do not work with probability and original
        order is influenced by the order of classes in 'constant' CLOUD_DETECTORS.
        """
        host_facts = {
            'virt.is_guest': True,
            'virt.host_type': 'kvm',
        }
        hw_facts = {
            'dmi.bios.vendor': 'Google',
            'dmi.bios.version': 'Google',
            'dmi.chassis.asset_tag': '7783-7084-3265-9085-8269-3286-77',
            'dmi.chassis.manufacturer': 'Microsoft'
        }
        self.host_fact_collector_instance.get_all.return_value = host_facts
        self.hw_fact_collector_instance.get_all.return_value = hw_facts
        detected_clouds = detect_cloud_provider()
        self.assertEqual(detected_clouds, ['gcp', 'azure', 'aws'])

    def test_collect_cloud_info_one_cloud_provider_detected(self):
        """
        Test the case, when we try to collect cloud info only for
        one detected cloud provider
        """
        self.aws_requests_mock.get = get_only_imds_v2_is_supported
        self.aws_requests_mock.put = put_imds_v2_token

        cloud_list = ['aws']
        cloud_info = collect_cloud_info(cloud_list)

        self.assertIsNotNone(cloud_info)
        self.assertTrue(len(cloud_info) > 0)
        self.assertTrue('cloud_id' in cloud_info)
        self.assertEqual(cloud_info['cloud_id'], 'aws')
        # Test metadata
        self.assertTrue('metadata' in cloud_info)
        b64_metadata = cloud_info['metadata']
        metadata = base64.b64decode(b64_metadata).decode('utf-8')
        self.assertEqual(metadata, AWS_METADATA)
        # Test signature
        self.assertTrue('signature' in cloud_info)
        b64_signature = cloud_info['signature']
        signature = base64.b64decode(b64_signature).decode('utf-8')
        self.assertEqual(
            signature,
            '-----BEGIN PKCS7-----\n' + AWS_SIGNATURE + '\n-----END PKCS7-----'
        )

    def test_collect_cloud_info_more_cloud_providers_detected(self):
        """
        Test the case, when we try to collect cloud info only for
        more than one cloud providers, because more than one cloud
        providers were detected
        """
        mock_result = Mock()
        mock_result.status_code = 400
        mock_result.text = 'Not Found 400'
        self.azure_requests_mock.get = Mock(return_value=mock_result)

        self.aws_requests_mock.get = get_only_imds_v2_is_supported
        self.aws_requests_mock.put = put_imds_v2_token

        cloud_list = ['azure', 'aws']
        cloud_info = collect_cloud_info(cloud_list)

        self.assertIsNotNone(cloud_info)
        self.assertTrue(len(cloud_info) > 0)
        self.assertTrue('cloud_id' in cloud_info)
        self.assertEqual(cloud_info['cloud_id'], 'aws')
        # Test metadata
        self.assertTrue('metadata' in cloud_info)
        b64_metadata = cloud_info['metadata']
        metadata = base64.b64decode(b64_metadata).decode('utf-8')
        self.assertEqual(metadata, AWS_METADATA)
        # Test signature
        self.assertTrue('signature' in cloud_info)
        b64_signature = cloud_info['signature']
        signature = base64.b64decode(b64_signature).decode('utf-8')
        self.assertEqual(
            signature,
            '-----BEGIN PKCS7-----\n' + AWS_SIGNATURE + '\n-----END PKCS7-----'
        )



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from .fixture import SubManFixture
from subscription_manager.overrides import Overrides, Override


class OverrideTests(SubManFixture):

    def setUp(self):
        SubManFixture.setUp(self)
        self.overrides = Overrides()

    def test_add_function(self):
        repos = ['x', 'y']
        override_props = {'a': 'b', 'c': 'd'}
        overrides = [Override(repo, name, value) for repo in repos for name, value in list(override_props.items())]
        expected = [
            {'contentLabel': 'x', 'name': 'a', 'value': 'b'},
            {'contentLabel': 'x', 'name': 'c', 'value': 'd'},
            {'contentLabel': 'y', 'name': 'a', 'value': 'b'},
            {'contentLabel': 'y', 'name': 'c', 'value': 'd'},
        ]
        result = self.overrides._add(overrides)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_function(self):
        repos = ['x', 'y']
        props_to_remove = ['a', 'b']
        removes = [Override(repo, name) for repo in repos for name in props_to_remove]
        expected = [
            {'contentLabel': 'x', 'name': 'a'},
            {'contentLabel': 'x', 'name': 'b'},
            {'contentLabel': 'y', 'name': 'a'},
            {'contentLabel': 'y', 'name': 'b'},
        ]
        result = self.overrides._remove(removes)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_all(self):
        repos = ['x', 'y']
        expected = [
            {'contentLabel': 'x'},
            {'contentLabel': 'y'},
        ]
        result = self.overrides._remove_all(repos)
        self.assertTrue(self.assert_items_equals(expected, result))

    def test_remove_all_with_no_repos_given(self):
        repos = []
        result = self.overrides._remove_all(repos)
        self.assertEqual(None, result)



# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
# Copyright (c) 2010 Ville Skyttä
# Copyright (c) 2009 Tim Lauridsen
# Copyright (c) 2007 Marcus Kuhn
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   James Antill <james@fedoraproject.org>
#   Marcus Kuhn
#   Toshio Kuratomi <toshio@fedoraproject.org>
#   Tim Lauridsen
#   Ville Skyttä
#
# Portions of this are from yum/i18n.py
# NOTE: originally from kitchen,
#   see https://github.com/fedora-infra/kitchen/blob/develop/kitchen2/kitchen/text/display.py
"""
-----------------------
Format Text for Display
-----------------------

Functions related to displaying unicode text.  Unicode characters don't all
have the same width so we need helper functions for displaying them.

.. versionadded:: 0.2 kitchen.display API 1.0.0
"""

# This is ported from ustr_utf8_* which I got from:
#     http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c
#  I've tried to leave it close to the original C (same names etc.) so that
# it is easy to read/compare both versions... James Antilles

#
# Reimplemented quite a bit of this for speed.  Use the bzr log or annotate
# commands to see what I've changed since importing this file.-Toshio Kuratomi

# ----------------------------- BEG utf8 ------------------to-----------
# This is an implementation of wcwidth() and wcswidth() (defined in
# IEEE Std 1002.1-2001) for Unicode.
#
# http://www.opengroup.org/onlinepubs/007904975/functions/wcwidth.html
# http://www.opengroup.org/onlinepubs/007904975/functions/wcswidth.html
#
# In fixed-width output devices, Latin characters all occupy a single
# "cell" position of equal width, whereas ideographic CJK characters
# occupy two such cells. Interoperability between terminal-line
# applications and (teletype-style) character terminals using the
# UTF-8 encoding requires agreement on which character should advance
# the cursor by how many cell positions. No established formal
# standards exist at present on which Unicode character shall occupy
# how many cell positions on character terminals. These routines are
# a first attempt of defining such behavior based on simple rules
# applied to data provided by the Unicode Consortium.
#
# [...]
#
# Markus Kuhn -- 2007-05-26 (Unicode 5.0)
#
# Permission to use, copy, modify, and distribute this software
# for any purpose and without fee is hereby granted. The author
# disclaims all warranties with regard to this software.
#
# Latest version: http://www.cl.cam.ac.uk/~mgk25/ucs/wcwidth.c


# Renamed but still pretty much JA's port of MK's code
def _interval_bisearch(value, table):
    """Binary search in an interval table.

    :arg value: numeric value to search for
    :arg table: Ordered list of intervals.  This is a list of two-tuples.  The
        elements of the two-tuple define an interval's start and end points.
    :returns: If :attr:`value` is found within an interval in the :attr:`table`
        return :data:`True`.  Otherwise, :data:`False`

    This function checks whether a numeric value is present within a table
    of intervals.  It checks using a binary search algorithm, dividing the
    list of values in half and checking against the values until it determines
    whether the value is in the table.
    """
    minimum = 0
    maximum = len(table) - 1
    if value < table[minimum][0] or value > table[maximum][1]:
        return False

    while maximum >= minimum:
        mid = divmod(minimum + maximum, 2)[0]
        if value > table[mid][1]:
            minimum = mid + 1
        elif value < table[mid][0]:
            maximum = mid - 1
        else:
            return True

    return False


_COMBINING = (
        (0x300, 0x36f), (0x483, 0x489), (0x591, 0x5bd),
        (0x5bf, 0x5bf), (0x5c1, 0x5c2), (0x5c4, 0x5c5),
        (0x5c7, 0x5c7), (0x600, 0x603), (0x610, 0x61a),
        (0x64b, 0x65f), (0x670, 0x670), (0x6d6, 0x6e4),
        (0x6e7, 0x6e8), (0x6ea, 0x6ed), (0x70f, 0x70f),
        (0x711, 0x711), (0x730, 0x74a), (0x7a6, 0x7b0),
        (0x7eb, 0x7f3), (0x816, 0x819), (0x81b, 0x823),
        (0x825, 0x827), (0x829, 0x82d), (0x859, 0x85b),
        (0x8d4, 0x8e1), (0x8e3, 0x8ff), (0x901, 0x902),
        (0x93c, 0x93c), (0x941, 0x948), (0x94d, 0x94d),
        (0x951, 0x954), (0x962, 0x963), (0x981, 0x981),
        (0x9bc, 0x9bc), (0x9c1, 0x9c4), (0x9cd, 0x9cd),
        (0x9e2, 0x9e3), (0xa01, 0xa02), (0xa3c, 0xa3c),
        (0xa41, 0xa42), (0xa47, 0xa48), (0xa4b, 0xa4d),
        (0xa70, 0xa71), (0xa81, 0xa82), (0xabc, 0xabc),
        (0xac1, 0xac5), (0xac7, 0xac8), (0xacd, 0xacd),
        (0xae2, 0xae3), (0xb01, 0xb01), (0xb3c, 0xb3c),
        (0xb3f, 0xb3f), (0xb41, 0xb43), (0xb4d, 0xb4d),
        (0xb56, 0xb56), (0xb82, 0xb82), (0xbc0, 0xbc0),
        (0xbcd, 0xbcd), (0xc3e, 0xc40), (0xc46, 0xc48),
        (0xc4a, 0xc4d), (0xc55, 0xc56), (0xcbc, 0xcbc),
        (0xcbf, 0xcbf), (0xcc6, 0xcc6), (0xccc, 0xccd),
        (0xce2, 0xce3), (0xd3b, 0xd3c), (0xd41, 0xd43),
        (0xd4d, 0xd4d), (0xdca, 0xdca), (0xdd2, 0xdd4),
        (0xdd6, 0xdd6), (0xe31, 0xe31), (0xe34, 0xe3a),
        (0xe47, 0xe4e), (0xeb1, 0xeb1), (0xeb4, 0xeb9),
        (0xebb, 0xebc), (0xec8, 0xecd), (0xf18, 0xf19),
        (0xf35, 0xf35), (0xf37, 0xf37), (0xf39, 0xf39),
        (0xf71, 0xf7e), (0xf80, 0xf84), (0xf86, 0xf87),
        (0xf90, 0xf97), (0xf99, 0xfbc), (0xfc6, 0xfc6),
        (0x102d, 0x1030), (0x1032, 0x1032), (0x1036, 0x1037),
        (0x1039, 0x103a), (0x1058, 0x1059), (0x108d, 0x108d),
        (0x1160, 0x11ff), (0x135d, 0x135f), (0x1712, 0x1714),
        (0x1732, 0x1734), (0x1752, 0x1753), (0x1772, 0x1773),
        (0x17b4, 0x17b5), (0x17b7, 0x17bd), (0x17c6, 0x17c6),
        (0x17c9, 0x17d3), (0x17dd, 0x17dd), (0x180b, 0x180d),
        (0x18a9, 0x18a9), (0x1920, 0x1922), (0x1927, 0x1928),
        (0x1932, 0x1932), (0x1939, 0x193b), (0x1a17, 0x1a18),
        (0x1a60, 0x1a60), (0x1a75, 0x1a7c), (0x1a7f, 0x1a7f),
        (0x1ab0, 0x1abd), (0x1b00, 0x1b03), (0x1b34, 0x1b34),
        (0x1b36, 0x1b3a), (0x1b3c, 0x1b3c), (0x1b42, 0x1b42),
        (0x1b44, 0x1b44), (0x1b6b, 0x1b73), (0x1baa, 0x1bab),
        (0x1be6, 0x1be6), (0x1bf2, 0x1bf3), (0x1c37, 0x1c37),
        (0x1cd0, 0x1cd2), (0x1cd4, 0x1ce0), (0x1ce2, 0x1ce8),
        (0x1ced, 0x1ced), (0x1cf4, 0x1cf4), (0x1cf8, 0x1cf9),
        (0x1dc0, 0x1df9), (0x1dfb, 0x1dff), (0x200b, 0x200f),
        (0x202a, 0x202e), (0x2060, 0x2063), (0x206a, 0x206f),
        (0x20d0, 0x20f0), (0x2cef, 0x2cf1), (0x2d7f, 0x2d7f),
        (0x2de0, 0x2dff), (0x302a, 0x302f), (0x3099, 0x309a),
        (0xa66f, 0xa66f), (0xa674, 0xa67d), (0xa69e, 0xa69f),
        (0xa6f0, 0xa6f1), (0xa806, 0xa806), (0xa80b, 0xa80b),
        (0xa825, 0xa826), (0xa8c4, 0xa8c4), (0xa8e0, 0xa8f1),
        (0xa92b, 0xa92d), (0xa953, 0xa953), (0xa9b3, 0xa9b3),
        (0xa9c0, 0xa9c0), (0xaab0, 0xaab0), (0xaab2, 0xaab4),
        (0xaab7, 0xaab8), (0xaabe, 0xaabf), (0xaac1, 0xaac1),
        (0xaaf6, 0xaaf6), (0xabed, 0xabed), (0xfb1e, 0xfb1e),
        (0xfe00, 0xfe0f), (0xfe20, 0xfe2f), (0xfeff, 0xfeff),
        (0xfff9, 0xfffb), (0x101fd, 0x101fd), (0x102e0, 0x102e0),
        (0x10376, 0x1037a), (0x10a01, 0x10a03), (0x10a05, 0x10a06),
        (0x10a0c, 0x10a0f), (0x10a38, 0x10a3a), (0x10a3f, 0x10a3f),
        (0x10ae5, 0x10ae6), (0x11046, 0x11046), (0x1107f, 0x1107f),
        (0x110b9, 0x110ba), (0x11100, 0x11102), (0x11133, 0x11134),
        (0x11173, 0x11173), (0x111c0, 0x111c0), (0x111ca, 0x111ca),
        (0x11235, 0x11236), (0x112e9, 0x112ea), (0x1133c, 0x1133c),
        (0x1134d, 0x1134d), (0x11366, 0x1136c), (0x11370, 0x11374),
        (0x11442, 0x11442), (0x11446, 0x11446), (0x114c2, 0x114c3),
        (0x115bf, 0x115c0), (0x1163f, 0x1163f), (0x116b6, 0x116b7),
        (0x1172b, 0x1172b), (0x11a34, 0x11a34), (0x11a47, 0x11a47),
        (0x11a99, 0x11a99), (0x11c3f, 0x11c3f), (0x11d42, 0x11d42),
        (0x11d44, 0x11d45), (0x16af0, 0x16af4), (0x16b30, 0x16b36),
        (0x1bc9e, 0x1bc9e), (0x1d165, 0x1d169), (0x1d16d, 0x1d182),
        (0x1d185, 0x1d18b), (0x1d1aa, 0x1d1ad), (0x1d242, 0x1d244),
        (0x1e000, 0x1e006), (0x1e008, 0x1e018), (0x1e01b, 0x1e021),
        (0x1e023, 0x1e024), (0x1e026, 0x1e02a), (0x1e8d0, 0x1e8d6),
        (0x1e944, 0x1e94a), (0xe0001, 0xe0001), (0xe0020, 0xe007f),
        (0xe0100, 0xe01ef), )


# Handling of control chars rewritten.  Rest is JA's port of MK's C code.
# -Toshio Kuratomi
# NOTE: Removed unused functionality (see note in docstring) - subscription-manager developers
def _ucp_width(ucs):
    """Get the :term:`textual width` of a ucs character

    :arg ucs: integer representing a single unicode :term:`code point`

    :returns: :term:`textual width` of the character.

    .. note::

        It's important to remember this is :term:`textual width` and not the
        number of characters or bytes.

        Some modifications were made when importing into subscription-manager; namely:
         - control character handling simplified
    """
    if _interval_bisearch(ucs, _COMBINING):
        # Combining characters return 0 width as they will be combined with
        # the width from other characters
        return 0

    if ucs <= 0x1f:
        return 0

    # if we arrive here, ucs is not a combining character

    return (1 +
      (ucs >= 0x1100 and
       (ucs <= 0x115f or                      # Hangul Jamo init. consonants
        ucs == 0x2329 or ucs == 0x232a or
        (ucs >= 0x2e80 and ucs <= 0xa4cf and
         ucs != 0x303f) or                    # CJK ... Yi
        (ucs >= 0xac00 and ucs <= 0xd7a3) or  # Hangul Syllables
        (ucs >= 0xf900 and ucs <= 0xfaff) or  # CJK Compatibility Ideographs
        (ucs >= 0xfe10 and ucs <= 0xfe19) or  # Vertical forms
        (ucs >= 0xfe30 and ucs <= 0xfe6f) or  # CJK Compatibility Forms
        (ucs >= 0xff00 and ucs <= 0xff60) or  # Fullwidth Forms
        (ucs >= 0xffe0 and ucs <= 0xffe6) or
        (ucs >= 0x20000 and ucs <= 0x2fffd) or
        (ucs >= 0x30000 and ucs <= 0x3fffd))))


# Wholly rewritten by me (LGPLv2+) -Toshio Kuratomi
# NOTE: Removed unused functionality (see note in docstring) - subscription-manager developers
def textual_width(msg):
    """Get the :term:`textual width` of a string

    :arg msg: :class:`str` string or byte :class:`bytes` to get the width of

    :returns: :term:`Textual width` of the :attr:`msg`.  This is the amount of
        space that the string will consume on a monospace display.  It's
        measured in the number of cell positions or columns it will take up on
        a monospace display.  This is **not** the number of glyphs that are in
        the string.

    .. note::

        This function can be wrong sometimes because Unicode does not specify
        a strict width value for all of the :term:`code points`.  In
        particular, we've found that some Tamil characters take up to four
        character cells but we return a lesser amount.

        Some modifications were made when importing into subscription-manager; namely:
         - simplified implementation to be both Python 2 and Python 3 compatible
         - control character handling simplified
         - we always pass unicode to the function, so no need to convert to unicode
    """
    # Add the width of each char
    return sum(_ucp_width(ord(char)) for char in msg)



from __future__ import print_function, division, absolute_import

#
# GUI Module for standalone subscription-manager cli
#
# Copyright (c) 2010 Red Hat, Inc.
#
# Authors: Pradeep Kilambi <pkilambi@redhat.com>
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
from subscription_manager.injection import require, IDENTITY, CERT_SORTER, CP_PROVIDER
import subscription_manager.injection as inj

import locale
import logging
import webbrowser
import os
import threading
import time
import socket

import rhsm.config as config
from rhsm import connection

from six.moves import urllib

from subscription_manager.ga import Gtk as ga_Gtk
from subscription_manager.ga import GLib as ga_GLib

from subscription_manager.branding import get_branding
from subscription_manager.entcertlib import EntCertActionInvoker
from subscription_manager.repolib import YumPluginManager
from rhsmlib.facts.hwprobe import ClassicCheck
from rhsmlib.services import unregister
from subscription_manager.utils import get_client_versions, get_server_versions, parse_baseurl_info, restart_virt_who, print_error

from rhsm import utils as rhsm_utils

from subscription_manager.gui import factsgui
from subscription_manager.gui import messageWindow
from subscription_manager.gui import networkConfig
from subscription_manager.gui import redeem
from subscription_manager.gui import registergui
from subscription_manager.gui import utils
from subscription_manager.gui import widgets

from subscription_manager.gui.about import AboutDialog
from subscription_manager.gui.allsubs import AllSubscriptionsTab
from subscription_manager.gui.importsub import ImportSubDialog
from subscription_manager.gui.installedtab import InstalledProductsTab
from subscription_manager.gui.mysubstab import MySubscriptionsTab
from subscription_manager.gui.preferences import PreferencesDialog
from subscription_manager.gui.utils import handle_gui_exception, linkify
from subscription_manager.gui.reposgui import RepositoriesDialog
from subscription_manager.gui.networkConfig import reset_resolver
from subscription_manager.overrides import Overrides
from subscription_manager.cli import system_exit

from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)

cfg = config.get_config_parser()

ONLINE_DOC_URL_TEMPLATE = "https://access.redhat.com/documentation/%s/red_hat_subscription_management/"
ONLINE_DOC_FALLBACK_URL = "https://access.redhat.com/documentation/en-us/red_hat_subscription_management/"

# every GUI browser from https://docs.python.org/2/library/webbrowser.html with updates within last 2 years of writing
PREFERRED_BROWSERS = [
    "mozilla",
    "firefox",
    "epiphany",
    "konqueror",
    "opera",
    "google-chrome",
    "chrome",
    "chromium",
    "chromium-browser",
]

# inform user of the URL in case our detection is outdated
NO_BROWSER_MESSAGE = _("Browser not detected. Documentation URL is %s.")


class Backend(object):
    """
    Wrapper for sharing UEP connections to Candlepin.

    Reference to a Backend object will be passed around UI components, so
    the UEP connection it contains can be modified/recreated and all
    components will have the updated connection.

    This also serves as a common wrapper for certificate directories and methods
    to monitor those directories for changes.
    """

    def __init__(self):
        self.identity = require(IDENTITY)
        self.cp_provider = require(CP_PROVIDER)

        self.update()

        self.product_dir = inj.require(inj.PROD_DIR)
        self.entitlement_dir = inj.require(inj.ENT_DIR)
        self.certlib = EntCertActionInvoker()
        self.overrides = Overrides()

        self.cs = require(CERT_SORTER)

    # make a create that does the init
    # and a update() for a name
    def update(self):
        self.create_uep()
        self.create_content_connection()

    def create_uep(self):
        # Re-initialize our connection:
        self.cp_provider.set_connection_info()

        # These objects hold a reference to the old uep and must be updated:
        # FIXME: We should find a way to update the connection so that the
        #        conncection objects are refreshed rather than recreated.

        self.certlib = EntCertActionInvoker()
        self.overrides = Overrides()

    def create_content_connection(self):
        (cdn_hostname, cdn_port, cdn_prefix) = parse_baseurl_info(cfg.get('rhsm', 'baseurl'))

        self.cp_provider.set_content_connection_info(cdn_hostname=cdn_hostname,
                                                     cdn_port=cdn_port)
        # ContentConnection now handles reading the proxy information
        return self.cp_provider.get_content_connection()

    # cause cert_sorter to cert_check to cause file_monitor.Monitor to look for
    # for new certs, add this as a main loop timer callback to do that on timer.
    def on_cert_check_timer(self):
        self.cs.force_cert_check()


class MainWindow(widgets.SubmanBaseWidget):
    """
    The new RHSM main window.
    """
    widget_names = ['main_window', 'notebook', 'system_name_label',
                    'register_menu_item', 'unregister_menu_item',
                    'redeem_menu_item', 'settings_menu_item', 'repos_menu_item',
                    'import_cert_menu_item']
    gui_file = "mainwindow"

    def log_server_version(self, uep):
        server_versions = get_server_versions(uep)
        log.debug("Server Versions: %s" % server_versions)
        # Remove this from the GTK main loop
        return False

    def _on_proxy_error_dialog_response(self, window, response):
        if response:
            self.network_config_dialog.show()
        else:
            system_exit(os.EX_UNAVAILABLE)

    def _exit(self, *args):
        system_exit(0)

    def __init__(self, backend=None,
                 ent_dir=None, prod_dir=None,
                 auto_launch_registration=False):
        super(MainWindow, self).__init__()

        rhsm_cfg = config.get_config_parser()
        proxy_server = rhsm_cfg.get("server", "proxy_hostname")
        proxy_port = int(rhsm_cfg.get("server", "proxy_port") or config.DEFAULT_PROXY_PORT)

        def show_proxy_error_dialog(proxy_auth_required=False):
            """
            When proxy server is set in configuration and it is not
            possible to connect to proxy server, then open dialog
            for setting proxy server.
            """
            if proxy_auth_required:
                proxy_user = rhsm_cfg.get("server", "proxy_user")
                proxy_password = rhsm_cfg.get("server", "proxy_password")
                if proxy_user or proxy_password:
                    err_msg = _("Wrong proxy username or password, please check your settings.")
                else:
                    err_msg = _("Proxy authentication required, please check your settings.")
            else:
                err_msg = _("Proxy connection failed, please check your settings.")
            print_error(err_msg)
            error_dialog = messageWindow.ContinueDialog(err_msg, self._get_window())
            error_dialog.connect("response", self._on_proxy_error_dialog_response)
            self.network_config_dialog = networkConfig.NetworkConfigDialog()
            # Sub-man gui will be terminated after saving settings and it is
            # necessary to start it once again.
            self.network_config_dialog.saveButton.connect("clicked", self._exit)
            self.network_config_dialog.cancelButton.connect("clicked", self._exit)

        self.backend = backend or Backend()
        cp = self.backend.cp_provider.get_consumer_auth_cp()

        # allow specifying no_proxy via api or config
        no_proxy = rhsm_cfg.get('server', 'no_proxy')
        if no_proxy:
            os.environ['no_proxy'] = no_proxy

        rhsm_utils.fix_no_proxy()
        log.debug('Environment variable NO_PROXY=%s will be used' % no_proxy)

        # Don't check the proxy server if the hostname we aim to connect to is covered by no_proxy
        if proxy_server and not urllib.request.proxy_bypass(rhsm_cfg.get('server', 'hostname')):
            if not utils.test_proxy_reachability(proxy_server, proxy_port):
                show_proxy_error_dialog()
                return

            try:
                # Try to send to the simplest Rest API call to Candlepin server.
                # This result will be used for getting version of Candlepin server.
                # See self.log_server_version.
                cp.supports_resource("status")
            except (socket.error, connection.ProxyException) as err:
                # See https://tools.ietf.org/html/rfc7235#section-4.3
                if "407 Proxy Authentication Required" in str(err.message):
                    show_proxy_error_dialog(proxy_auth_required=True)
                    return

        self.identity = require(IDENTITY)
        log.debug("Client Versions: %s " % get_client_versions())
        ga_GLib.idle_add(self.log_server_version, cp)

        settings = self.main_window.get_settings()

        # prevent gtk from trying to save a list of recently used files, which
        # as root, causes gtk warning:
        #  "Attempting to set the permissions of `/root/.local/share/recently-used.xbel'
        # The __name__ use is just for the 'origin' value gtk uses to store
        # where a Gtk.Settings value was set.
        settings.set_long_property('gtk-recent-files-max-age', 0,
                                   "%s:%s" % (__name__, type(self).__name__))

        ga_Gtk.Window.set_default_icon_name("subscription-manager")

        self.product_dir = prod_dir or self.backend.product_dir
        self.entitlement_dir = ent_dir or self.backend.entitlement_dir

        self.system_facts_dialog = factsgui.SystemFactsDialog(update_callback=self._handle_facts_updated)

        self.preferences_dialog = PreferencesDialog(self.backend,
                                                    self._get_window())

        self.repos_dialog = RepositoriesDialog(self.backend, self._get_window())

        self.import_sub_dialog = ImportSubDialog()

        self.network_config_dialog = networkConfig.NetworkConfigDialog()
        self.network_config_dialog.saveButton.connect("clicked", self._config_changed)

        self.redeem_dialog = redeem.RedeemDialog(self.backend)

        self.installed_tab_icon = ga_Gtk.Image()
        self.installed_tab_icon.set_from_stock(ga_Gtk.STOCK_YES,
                ga_Gtk.IconSize.MENU)

        self.installed_tab = InstalledProductsTab(self.backend,
                                                  self.installed_tab_icon,
                                                  self,
                                                  ent_dir=self.entitlement_dir,
                                                  prod_dir=self.product_dir)

        self.my_subs_tab = MySubscriptionsTab(self.backend,
                                              self.main_window,
                                              ent_dir=self.entitlement_dir,
                                              prod_dir=self.product_dir)

        self.all_subs_tab = AllSubscriptionsTab(self.backend,
                                                self.main_window)

        hbox = ga_Gtk.HBox(spacing=6)
        hbox.pack_start(self.installed_tab_icon, False, False, 0)
        hbox.pack_start(ga_Gtk.Label(self.installed_tab.get_label()), False, False, 0)
        self.notebook.append_page(self.installed_tab.get_content(), hbox)
        hbox.show_all()

        self.notebook.append_page(self.my_subs_tab.get_content(),
                                  ga_Gtk.Label(self.my_subs_tab.get_label()))

        self.connect_signals({
            "on_register_menu_item_activate": self._register_item_clicked,
            "on_unregister_menu_item_activate": self._unregister_item_clicked,
            "on_import_cert_menu_item_activate": self._import_cert_item_clicked,
            "on_view_facts_menu_item_activate": self._facts_item_clicked,
            "on_proxy_config_menu_item_activate": self._proxy_config_item_clicked,
            "on_redeem_menu_item_activate": self._redeem_item_clicked,
            "on_preferences_menu_item_activate": self._preferences_item_clicked,
            "on_repos_menu_item_activate": self._repos_item_clicked,
            "on_about_menu_item_activate": self._about_item_clicked,
            "on_getting_started_menu_item_activate": self._getting_started_item_clicked,
            "on_online_docs_menu_item_activate": self._online_docs_item_clicked,
            "on_quit_menu_item_activate": ga_Gtk.main_quit,
        })

        # various state tracking for async operations
        self._show_overrides = False
        self._can_redeem = False

        self.backend.cs.add_callback(self.on_cert_sorter_cert_change)

        self.main_window.show_all()

        # Check to see if already registered to old RHN/Spacewalk
        # and show dialog if so
        self._check_rhn_classic()

        # Update everything with compliance data
        self.backend.cs.notify()

        # managergui needs cert_sort.cert_monitor.run_check() to run
        # on a timer to detect cert changes from outside the gui
        # (via manual provisioning).
        cert_monitor_thread = threading.Thread(target=self._cert_check_timer, name="CertMonitorThread")
        cert_monitor_thread.daemon = True
        cert_monitor_thread.start()

        if auto_launch_registration and not self.registered():
            self._register_item_clicked(None)

        enabled_yum_plugins = YumPluginManager.enable_pkg_plugins()
        if len(enabled_yum_plugins) > 0:
            messageWindow.InfoDialog(
                YumPluginManager.warning_message(enabled_yum_plugins),
                self._get_window(),
                _("Warning - subscription-manager plugins were automatically enabled")
            )

    def registered(self):
        return self.identity.is_valid()

    def _cert_check_timer(self):
        while True:
            self.backend.on_cert_check_timer()
            time.sleep(2.0)

    def _cert_change_update(self):
        # Update installed products
        self.installed_tab.refresh()
        # Update attached subs
        self.my_subs_tab.refresh()
        # Update main window
        self.refresh()
        # Reset repos dialog, see bz 1132919
        self.repos_dialog = RepositoriesDialog(self.backend, self._get_window())

    # When something causes cert_sorter to update it's state, refresh the gui
    # The cert directories being updated will cause this (either noticed
    # from a timer, or via cert_sort.force_cert_check).
    def on_cert_sorter_cert_change(self):
        # gather data used in GUI refresh
        self._show_overrides = self._should_show_overrides()
        self._can_redeem = self._should_show_redeem()
        self.installed_tab.update()
        self.my_subs_tab.update_subscriptions(update_gui=False)  # don't update GUI since we're in a different thread

        # queue up in the main thread since the cert check may be done by another thread.
        ga_GLib.idle_add(self._cert_change_update)

    def _on_sla_back_button_press(self):
        self._perform_unregister()
        self._register_item_clicked(None)

    def _on_sla_cancel_button_press(self):
        self._perform_unregister()

    def on_registration_changed(self):
        # Show the All Subscriptions tab if registered, hide it otherwise:
        if self.registered() and self.notebook.get_n_pages() == 2:
            self.notebook.append_page(self.all_subs_tab.get_content(),
                    ga_Gtk.Label(self.all_subs_tab.get_label()))
        elif not self.registered() and self.notebook.get_n_pages() == 3:
            self.notebook.set_current_page(0)
            self.notebook.remove_page(2)

        # we've unregistered, clear pools from all subscriptions tab
        # so it's correct if we reshow it
        self.all_subs_tab.sub_details.clear()
        self.all_subs_tab.clear_pools()

        self.installed_tab.set_registered(self.registered())

        self._show_buttons()
        self._show_redemption_buttons()

    def refresh(self):
        """ Refresh the UI. """
        # Always run on startup, when there is no last_uuid
        if not hasattr(self, 'last_uuid') or self.identity.uuid != self.last_uuid:
            self.last_uuid = self.identity.uuid
            self.on_registration_changed()

        self.all_subs_tab.refresh()
        self.installed_tab.refresh()
        self.my_subs_tab.refresh()

    def _get_window(self):
        """
        Return the window containing this widget (might be different for
        firstboot).
        """
        return self.main_window

    def _show_buttons(self):
        """
        Renders the Tools buttons dynamically.
        """
        is_registered = self.registered()
        if is_registered:
            #self.register_menu_item.hide()
            self.register_menu_item.set_sensitive(False)
            self.unregister_menu_item.set_sensitive(True)
            self.settings_menu_item.set_sensitive(True)  # preferences
            self.import_cert_menu_item.set_sensitive(False)
        else:
            self.register_menu_item.set_sensitive(True)
            self.unregister_menu_item.set_sensitive(False)
            self.settings_menu_item.set_sensitive(False)
            self.import_cert_menu_item.set_sensitive(True)
        if self._show_overrides:
            self.repos_menu_item.set_sensitive(True)
        else:
            self.repos_menu_item.set_sensitive(False)

    def _should_show_overrides(self):
        is_registered = self.registered()

        show_overrides = False
        try:
            cp = self.backend.cp_provider.get_consumer_auth_cp()
            # This can throw an exception if we cannot connect to the server, bz 1058374
            show_overrides = is_registered and cp.supports_resource('content_overrides')
        except Exception as e:
            log.debug("Failed to check if the server supports resource content_overrides")
            log.debug(e)

        return show_overrides

    def _show_redemption_buttons(self):
        if self._can_redeem:
            self.redeem_menu_item.set_sensitive(True)
        else:
            self.redeem_menu_item.set_sensitive(False)

    def _should_show_redeem(self):
        # Check if consumer can redeem a subscription - if an identity cert exists
        can_redeem = False

        if self.identity.uuid:
            try:
                consumer = self.backend.cp_provider.get_consumer_auth_cp().getConsumer(self.identity.uuid, None, None)
                can_redeem = consumer['canActivate']
            except Exception:
                can_redeem = False

        return can_redeem

    def _register_item_clicked(self, widget):
        registration_dialog = registergui.RegisterDialog(self.backend)
        registration_dialog.register_dialog.connect('destroy',
                                                    self._on_dialog_destroy,
                                                    widget)
        registration_dialog.window.set_transient_for(self._get_window())

        if registration_dialog and widget:
            widget.set_sensitive(False)

        registration_dialog.initialize()
        registration_dialog.show()

    def _on_dialog_destroy(self, obj, widget):
        # bz#1382897 make sure register menu item is left in appropriate state
        if (widget is not self.register_menu_item or not self.registered()) and widget:
            widget.set_sensitive(True)
        return False

    def _preferences_item_clicked(self, widget):
        try:
            self.preferences_dialog.show()
        except Exception as e:
            handle_gui_exception(e, _("Error in preferences dialog."
                                      "Please see /var/log/rhsm/rhsm.log for more information."),
                                 self._get_window())

    def _repos_item_clicked(self, widget):
        try:
            self.repos_dialog.show()
        except Exception as e:
            handle_gui_exception(e, _("Error in repos dialog. "
                                      "Please see /var/log/rhsm/rhsm.log for more information."),
                                 self._get_window())

    def _on_unregister_prompt_response(self, dialog, response):
        if not response:
            log.debug("unregister prompt not confirmed. cancelling")
            return
        log.debug("Proceeding with un-registration: %s", self.identity.uuid)
        self._perform_unregister()

    def _perform_unregister(self):
        try:
            reset_resolver()
            unregister.UnregisterService(self.backend.cp_provider.get_consumer_auth_cp()).unregister()
        except Exception as e:
            log.error("Error unregistering system with entitlement platform.")
            handle_gui_exception(e, _("<b>Errors were encountered during unregister.</b>") +
                                      "\n%s\n" +
                                      _("Please see /var/log/rhsm/rhsm.log for more information."),
                                self.main_window,
                                log_msg="Consumer may need to be manually cleaned up: %s" %
                                self.identity.uuid)
        # managerlib.unregister removes product and entitlement directories
        self.backend.product_dir.__init__()
        self.backend.entitlement_dir.__init__()

        # We have new credentials, restart virt-who
        restart_virt_who()

        self.backend.cs.force_cert_check()

    def _unregister_item_clicked(self, widget):
        log.debug("Unregister button pressed, asking for confirmation.")
        prompt = messageWindow.YesNoDialog(
                _("<b>Are you sure you want to unregister?</b>"),
                self._get_window())
        prompt.connect('response', self._on_unregister_prompt_response)

    def _proxy_config_item_clicked(self, widget):
        self.network_config_dialog.set_parent_window(self._get_window())
        self.network_config_dialog.show()

    def _facts_item_clicked(self, widget):
        self.system_facts_dialog.set_parent_window(self._get_window())
        self.system_facts_dialog.show()

    def _import_cert_item_clicked(self, widget):
        self.import_sub_dialog.set_parent_window(self._get_window())
        self.import_sub_dialog.show()

    def _update_certificates_button_clicked(self, widget):
        autobind_wizard = registergui.AutobindWizardDialog(self.backend)
        autobind_wizard.register_dialog.connect('destroy',
                                                self._on_dialog_destroy,
                                                widget)
        autobind_wizard.window.set_transient_for(self._get_window())

        if autobind_wizard and widget:
            widget.set_sensitive(False)

        autobind_wizard.initialize()
        autobind_wizard.show()

    def _redeem_item_clicked(self, widget):
        self.redeem_dialog.set_parent_window(self._get_window())
        self.redeem_dialog.show()

    def _getting_started_item_clicked(self, widget):
        try:
            # try to open documentation in yelp
            ga_Gtk.show_uri(None, 'ghelp:subscription-manager', time.time())
        except Exception as e:
            # if we can't open it, it's probably because the user didn't
            # install the docs, or yelp. no need to bother them.
            log.warn("Unable to open help documentation: %s", e)

    def _about_item_clicked(self, widget):
        about = AboutDialog(self._get_window(), self.backend)
        about.show()

    def _online_docs_item_clicked(self, widget):
        browser = None
        for possible_browser in PREFERRED_BROWSERS:
            try:
                browser = webbrowser.get(possible_browser)
                break
            except webbrowser.Error:
                pass
        if browser is None:
            utils.show_error_window(NO_BROWSER_MESSAGE % (self._get_online_doc_url()))
        else:
            webbrowser.open_new(self._get_online_doc_url())

    def _quit_item_clicked(self):
        self.hide()

    def _config_changed(self, widget):
        # update the backend's UEP in case we changed proxy
        # config. We specify all these settings since they
        # are new and the default UEP init won't get them
        # (it's default args are set at class init time)
        self.backend.update()

    def _check_rhn_classic(self):
        if ClassicCheck().is_registered_with_classic():
            prompt = messageWindow.ContinueDialog(
                    linkify(get_branding().REGISTERED_TO_OTHER_WARNING),
                    self.main_window, _("System Already Registered"))
            prompt.connect('response', self._on_rhn_classic_response)

    def _on_rhn_classic_response(self, dialog, response):
        if not response:
            self.main_window.hide()

    def _get_online_doc_url(self):
        lang, encoding = locale.getdefaultlocale()
        if lang is not None:
            url = ONLINE_DOC_URL_TEMPLATE % (lang.replace("_", "-").lower())
        else:
            url = ONLINE_DOC_FALLBACK_URL
        try:
            urllib.request.urlopen(url)
        except urllib.error.URLError:
            # Use the default if there is no translation.
            url = ONLINE_DOC_FALLBACK_URL
        return url

    def _handle_facts_updated(self):
        # see bz 1323271 - update compliance on update of facts
        self.backend.cs.load()
        self.backend.cs.notify()



from __future__ import print_function, division, absolute_import

# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

#from gi.repository import Gtk
from datetime import datetime, timedelta
from rhsm.certificate import GMT

from subscription_manager.ga import Gtk as ga_Gtk
from subscription_manager.gui.storage import MappedTreeStore
from subscription_manager.gui.widgets import MachineTypeColumn, QuantitySelectionColumn, \
                                             SubDetailsWidget, ContractSubDetailsWidget, \
                                             DatePicker, HasSortableWidget
from dateutil.tz import tzlocal
from nose.plugins.attrib import attr


@attr('gui')
class TestSubDetailsWidget(unittest.TestCase):
    widget = SubDetailsWidget
    sku_text = "Some SKU"
    expected_sub_text = "All Available Subscription Text"

    def show(self, details):
        details.show(name="Some Product", contract="123123",
                     start=datetime.now(GMT()), end=datetime.now(GMT()) + timedelta(days=365),
                     highlight="some filter", support_level="Standard",
                     support_type="L1",
                     sku=self.sku_text)

    def test_show(self):
        details = self.widget(None)
        self.show(details)

    def test_clear(self):
        details = self.widget(None)
        self.show(details)
        details.clear()

    def test_a11y(self):
        details = self.widget(None)
        self.show(details)
        sub_text = details.subscription_text.get_accessible().get_name()
        self.assertEqual(self.expected_sub_text, sub_text)


@attr('gui')
class TestContractSubDetailsWidget(TestSubDetailsWidget):
    widget = ContractSubDetailsWidget
    expected_sub_text = "Subscription Text"

    def test_get_expired_bg(self):
        details = self.widget(None)
        self.show(details)
        yesterday = datetime.now(GMT()) - timedelta(days=1)
        bg_color = details._get_date_bg(yesterday, True)
        self.assertEqual(details.expired_color, bg_color)

    def test_get_warning_bg(self):
        details = self.widget(None)
        self.show(details)
        tomorrow = datetime.now(GMT()) + timedelta(days=1)
        bg_color = details._get_date_bg(tomorrow, True)
        self.assertEqual(details.warning_color, bg_color)

    def test_get_details(self):
        details = self.widget(None)
        reasons = ['reason 1', 'reason 2']
        details.show("Some Product", reasons=reasons, start=datetime.now(GMT()), end=datetime.now(GMT()) + timedelta(days=365))
        buff = details.details_view.get_buffer()
        result_list = buff.get_text(buff.get_bounds()[0],
                                    buff.get_bounds()[1],
                                    include_hidden_chars=False).split("\n")
        self.assertEqual(reasons, result_list)

    def testVirtOnly(self):
        details = self.widget(None)
        self.show(details)
        d = datetime(2011, 4, 16, tzinfo=tzlocal())
        start_date = datetime(d.year, d.month, d.day, tzinfo=tzlocal())
        end_date = datetime(d.year + 1, d.month, d.day, tzinfo=tzlocal())
        details.show('noname', contract='c', start=start_date, end=end_date, account='a',
                     management='m', support_level='s_l',
                     support_type='s_t', virt_only='v_o')
        s_iter = details.virt_only_text.get_buffer().get_start_iter()
        e_iter = details.virt_only_text.get_buffer().get_end_iter()
        self.assertEqual(details.virt_only_text.get_buffer().get_text(s_iter, e_iter, False), 'v_o')


@attr('gui')
class TestDatePicker(unittest.TestCase):
    def test_date_picker_date(self):
        d = datetime(2033, 12, 29, tzinfo=tzlocal())
        self._assert_is_isoformat(d)

    def test_date_validate_2000_12_1(self):
        d = datetime(2000, 12, 1, tzinfo=tzlocal())
        self._assert_is_isoformat(d)

    def test_date_validate_2000_1_22(self):
        d = datetime(2000, 1, 12, tzinfo=tzlocal())
        self._assert_is_isoformat(d)

    def test_date_validate_1_1_2000(self):
        d = datetime(2000, 1, 1, tzinfo=tzlocal())
        self._assert_is_isoformat(d)

    # why? because some locales fail to parse in dates with
    # double digt months
    def test_date_validate_12_29_2020(self):
        #with Capture(silent=True):
        d = datetime(2020, 12, 29, tzinfo=tzlocal())
        self._assert_is_isoformat(d)

    def _assert_is_isoformat(self, d):
        date_picker = DatePicker(d)
        valid = date_picker.date_entry_validate()
        self.assertTrue(valid)
        self.assertEqual(date_picker._date_entry.get_text(), d.date().isoformat())


class BaseColumnTest(unittest.TestCase):

    def _assert_column_value(self, column_class, model_bool_val, expected_text):
        model = ga_Gtk.ListStore(bool)
        model.append([model_bool_val])

        column = column_class(0)
        column._render_cell(None, column.renderer, model, model.get_iter_first())
        self.assertEqual(expected_text, column.renderer.get_property("text"))


@attr('gui')
class TestHasSortableWidget(unittest.TestCase):

    def _run_cases(self, cases, expected):
        for index, case in enumerate(cases):
            result = HasSortableWidget.compare_text(*case)
            self.assertEqual(result, expected[index])

    def test_compare_text_ints(self):
        # Two string representations of ints
        str1 = '1'
        str2 = '2'
        cases = [
            (str1, str2),  # x < y should return -1
            (str2, str1),  # x > y should return 1
            (str1, str1)  # x == y should return 0
        ]
        expected = [
            -1,
            1,
            0
        ]

        self._run_cases(cases, expected)

    def test_compare_text_unlimited(self):
        # Test unlimited comparison
        unlimited = 'Unlimited'
        str1 = '1'
        cases = [
            (unlimited, str1),  # Unlimited, 1 should return 1
            (str1, unlimited),  # 1, Unlimited should return -1
            (unlimited, unlimited)  # Unlimited, Unlimited should return 0
        ]
        expected = [
            1,
            -1,
            0
        ]

        self._run_cases(cases, expected)

    def test_compare_alphabetic_text(self):
        cases = [
            ('a', 'b'),
            ('b', 'a'),
            ('a', 'a')
        ]

        def _cmp(x1, x2):
            if x1 < x2:
                return -1
            elif x1 == x2:
                return 0
            else:
                return 1

        expected = [_cmp(*case) for case in cases]

        self._run_cases(cases, expected)


@attr('gui')
class TestMachineTypeColumn(BaseColumnTest):

    def test_render_virtual_when_virt_only(self):
        self._assert_column_value(MachineTypeColumn, True,
                                  MachineTypeColumn.VIRTUAL_MACHINE)

    def test_render_physical_when_not_virt_only(self):
        self._assert_column_value(MachineTypeColumn, False,
                                   MachineTypeColumn.PHYSICAL_MACHINE)


@attr('gui')
class TestQuantitySelectionColumnTests(unittest.TestCase):

    def test__update_cell_based_on_data_clears_cell_when_row_has_children(self):
        column, tree_model, tree_iter = self._setup_column(1, False)
        tree_model.add_map(tree_iter, self._create_store_map(1, False, 15, 2))

        column.quantity_renderer.set_property("text", "22")
        column._update_cell_based_on_data(None, column.quantity_renderer, tree_model, tree_iter)

        self.assertEqual("", column.quantity_renderer.get_property("text"))

    def test_update_cell_based_on_data_does_not_clear_cell_when_row_has_no_children(self):
        column, tree_model, tree_iter = self._setup_column(1, False)

        # Manually set the text value here to make sure that the value is not reset.
        column.quantity_renderer.set_property("text", "12")
        column._update_cell_based_on_data(None, column.quantity_renderer, tree_model, tree_iter)

        self.assertNotEqual("", column.quantity_renderer.get_property("text"))

    def test_editor_is_disabled_when_not_multi_entitlement(self):
        is_multi_entitled = False
        column, tree_model, tree_iter = self._setup_column(1, is_multi_entitled)
        column._update_cell_based_on_data(None, column.quantity_renderer, tree_model, tree_iter)
        self.assertEqual(is_multi_entitled, column.quantity_renderer.get_property("editable"))

    def test_editor_is_enabled_when_multi_entitlement(self):
        is_multi_entitled = True
        column, tree_model, tree_iter = self._setup_column(1, is_multi_entitled)
        column._update_cell_based_on_data(None, column.quantity_renderer, tree_model, tree_iter)
        self.assertEqual(is_multi_entitled, column.quantity_renderer.get_property("editable"))

    def test_value_not_changed_when_editor_has_invalid_text(self):
        expected_initial_value = 12
        column, tree_model, tree_iter = self._setup_column(expected_initial_value, False)
        column._on_edit(column.quantity_renderer, tree_model.get_path(tree_iter), "aaa",
                        tree_model)
        self.assertEqual(expected_initial_value,
                          tree_model.get_value(tree_iter, column.quantity_store_idx))

    def test_value_changed_when_editor_has_valid_text(self):
        column, tree_model, tree_iter = self._setup_column(1, False)
        column._on_edit(column.quantity_renderer, tree_model.get_path(tree_iter), "20",
                        tree_model)
        self.assertEqual(20, tree_model.get_value(tree_iter, column.quantity_store_idx))

    def test_filter_spinner_value_only_allows_digits(self):
        self._run_filter_value_test("4", True)
        self._run_filter_value_test("q", False)
        self._run_filter_value_test("!", False)
        self._run_filter_value_test("1d3f", False)

    def test_filter_spinner_value_always_allows_zero(self):
        self._run_filter_value_test("0", True, upper=5, lower=1)

    # Tests for values like 01, 002, 00003 ...
    def test_filter_spinner_value_does_not_allow_zero_at_start_of_value(self):
        self._run_filter_value_test("02", False)
        self._run_filter_value_test("004", False)
        self._run_filter_value_test("0", True)

    def test_filter_spinner_value_does_not_accept_value_over_upper_limit(self):
        self._run_filter_value_test("13", False, upper=12.0)

    def test_filter_spinner_value_allows_value_on_bounds(self):
        self._run_filter_value_test("1", True, upper=10, lower=1)
        self._run_filter_value_test("10", True, upper=10, lower=1)

    def _run_filter_value_test(self, test_input_value, is_allowed, upper=15, lower=1, step_incr=1):
        column, tree_model, tree_iter = self._setup_column(1, True)

        adjustment = ga_Gtk.Adjustment(upper=upper, lower=lower, value=7.0, step_incr=step_incr)
        # Simulate the editable created by the CellRendererSpin object.
        editable = ga_Gtk.SpinButton()
        editable.set_property("adjustment", adjustment)

        self.stopped = False

        def ensure_stopped(name):
            self.stopped = True

        editable.emit_stop_by_name = ensure_stopped

        column._filter_spinner_value("test-event", editable, test_input_value)

        self.assertEqual(not is_allowed, self.stopped)

    def _create_store_map(self, quantity, multi_entitlement, available, increment):
        return {"quantity": quantity, "multi-entitlement": multi_entitlement, "available_store": available,
                "quantity_increment": increment}

    def _setup_column(self, initial_quantity, inital_multi_entitlement, initial_available=15,
            initial_increment=2):
        tree_model = MappedTreeStore(self._create_store_map(int, bool, int, int))
        column = QuantitySelectionColumn("test-col", tree_model, tree_model['quantity'],
                                         tree_model['multi-entitlement'], tree_model['available_store'],
                                         tree_model['quantity_increment'])
        tree_iter = tree_model.add_map(None, self._create_store_map(initial_quantity,
                                                               inital_multi_entitlement, initial_available,
                                                               initial_increment))
        return (column, tree_model, tree_iter)

    def test_increment_based_on_provided_data(self):
        (column, tree_model, tree_iter) = self._setup_column(15, True, 15, 2)
        column._update_cell_based_on_data(column, column.quantity_renderer, tree_model, tree_iter)
        adj = column.quantity_renderer.get_property("adjustment")
        self.assertEqual(2, int(adj.get_property("step-increment")))
        self.assertEqual(2, int(adj.get_property("lower")))

    def test_on_edit_refuses_non_multiples_of_increment(self):
        (column, tree_model, tree_iter) = self._setup_column(10, True, 15, 2)
        column._on_edit(None, 0, 7, tree_model)
        self.assertEqual(10, int(tree_model.get_value(tree_iter, tree_model['quantity'])))

    def test_on_edit_allows_multiples_of_increment(self):
        (column, tree_model, tree_iter) = self._setup_column(10, True, 15, 2)
        column._on_edit(None, 0, 8, tree_model)
        self.assertEqual(8, tree_model.get_value(tree_iter, tree_model['quantity']))

    def test_on_edit_refuses_value_lower_than_increment(self):
        (column, tree_model, tree_iter) = self._setup_column(10, True, 15, 4)
        column._on_edit(None, 0, 2, tree_model)
        self.assertEqual(10, tree_model.get_value(tree_iter, tree_model['quantity']))



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

from subscription_manager import injection as inj
from subscription_manager import certlib


class InstalledProductsActionInvoker(certlib.BaseActionInvoker):
    """Used by rhsmcertd to update the installed
    products on this system periodically.
    """
    def _do_update(self):
        action = InstalledProductsActionCommand()
        return action.perform()


class InstalledProductsActionCommand(object):
    """Update the consumers installed product list to RHSM API.

    Returns a InstalledProductsActionReport.
    """
    def __init__(self):
        self.report = InstalledProductsActionReport()
        self.cp_provider = inj.require(inj.CP_PROVIDER)
        self.uep = self.cp_provider.get_consumer_auth_cp()

    def perform(self):
        mgr = inj.require(inj.INSTALLED_PRODUCTS_MANAGER)
        consumer_identity = inj.require(inj.IDENTITY)

        ret = mgr.update_check(self.uep, consumer_identity.uuid)
        self.report._status = ret
        return self.report


class InstalledProductsActionReport(certlib.ActionReport):
    name = "Installed Products"



from __future__ import print_function, division, absolute_import

#
# Read and parse /proc/cpuinfo
#
# Copyright (c) 2015 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# needs to be able to provide a data object for each cpu blob and
# for the system
#
# needs to be able to aggregate multiple cpu data objects and create
# an exemplar cpuinfo  (ie, if we want to ignore cpus at different
# speeds, this may be where)
#
# needs to work non-root if possible, possibly as a standalone module/cli
#
# Expect the info available in cpuinfo to very across arches, and across
# kernel and cpu versions. Some arches have almost no info. Some have tons.
# Some provide hex values, most decimal.
#
# Expect the field names to change often. Don't expect field names to
# be unique.
#
# Expect some fields to disappear without warning at any oppurtunity
# (This includes changing arch, version, kernel, os vendor. It also includes
#  no reason at all. cpus can disappear. cpus can remove fields. they can
#  reappear).
#
# Expect values of cpuinfo fields to change, somethings constantly. cpu speed
# for example, can actually vary _every_ time it is read.
#
# GOTCHAS: the field names are non consistent across arches, and can conflict
#          semantically.
#
#         surprise, some are not even one key: value per line (see s390x)
#
# context manager?
# class CpuinfoFile()
#     .read()
#  handle io errors
#
# can take file like object or
# class BaseParseCpuInfo()
#
# class FieldNameCanonicalizer()
#  ie, convert 'model name' to model_name
#  and 'proccesor' processor
#  and 'Processor' to... seriously aarch64?
#    ('Processor' and 'processor' fields...)
#
# class CpuInfo() the main interface class
#     arch = None
#     cpuinfo_class = None
#     # avoid, count of cpus/sockets/etc
#
#
# class X86_64():
#
# class S390X():
#   with fun "multiple values per line"
#
# class Aarch64():
#   with hex values and a system stanza
#
# class Ppc64():
#    system stanza and system model
# factory to init proper one based... uname.machine? 'arch' file?

import collections
import itertools
import logging
import os

# mostly populated from the arm CPUID instruction
# http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.ddi0432c/Bhccjgga.html
# http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.ddi0432c/Chdeaeij.html
#
# aarch "version" info
# CPU implementer : 0x50
# CPU architecture: AArch64
# CPU variant : 0x0
# CPU part    : 0x000
# CPU revision    : 0

# Mostly info from intel CPUID instruction
# http://en.wikipedia.org/wiki/CPUID
#
# intel "version" info
# processor   : 22
# vendor_id   : GenuineIntel
# cpu family  : 6
# model       : 45
# model name  : Intel(R) Xeon(R) CPU E5-2630 0 @ 2.30GHz
# stepping    : 7
# microcode   : 0x710

log = logging.getLogger('rhsm-app.' + __name__)


class DefaultCpuFields(object):
    """Maps generic cpuinfo fields to the corresponding field from ProcessorModel.

    For, a cpu MODEL (a number or string that the cpu vendor assigns to that model of
    cpu, '45' for an intel Xeon for example)
    is in the 'model' field in /proc/cpuinfo, and the
    'model' in the sluggified field in X86_64ProcessorModel. For aarch64,
    the field 'cpu_part' is it's MODEL.
    """
    MODEL_NAME = "model_name"
    MODEL = "model"


class X86_64Fields(object):
    MODEL_NAME = 'model_name'
    MODEL = 'model'


class Aarch64Fields(object):
    MODEL = 'cpu_part'
    MODEL_NAME = 'model_name'


class Ppc64Fields(object):
    MODEL = 'model'
    MODEL_NAME = 'machine'


# represent the data in /proc/cpuinfo, which may include multiple processors
class CpuinfoModel(object):
    fields_class = DefaultCpuFields

    def __init__(self, cpuinfo_data=None):
        # The contents of /proc/cpuinfo
        self.cpuinfo_data = cpuinfo_data

        # A iterable of CpuInfoModels, one for each processor in cpuinfo
        self.processors = []

        # prologues or footnotes not associated with a particular processor
        self.other = []

        # If were going to pretend all the cpus are the same,
        # what do they all have in common.
        self.common = {}

        # model name    : Intel(R) Core(TM) i5 CPU       M 560  @ 2.67GHz
        self._model_name = None

        # a model number
        # "45" for intel processor example above
        self._model = None

    @property
    def count(self):
        return len(self.processors)

    @property
    def model_name(self):
        if self._model_name:
            return self._model_name

        if not self.common:
            return None

        return self.common.get(self.fields_class.MODEL_NAME, None)

    @property
    def model(self):
        if self._model:
            return self._model

        if not self.common:
            return None

        return self.common.get(self.fields_class.MODEL, None)

    def __str__(self):
        lines = []
        lines.append("Processor count: %s" % self.count)
        lines.append('model_name: %s' % self.model_name)
        lines.append("")
        for k in sorted(self.common.keys()):
            lines.append("%s: %s" % (k, self.common[k]))
        lines.append("")
        for k, v in self.other:
            lines.append("%s: %s" % (k, v))
        lines.append("")
        return "\n".join(lines)


class Aarch64ProcessorModel(dict):
    "The info corresponding to the info about each aarch64 processor entry in cpuinfo"
    pass


class X86_64ProcessorModel(dict):
    "The info corresponding to the info about each X86_64 processor entry in cpuinfo"
    pass


class Ppc64ProcessorModel(dict):
    "The info corresponding to the info about each ppc64 processor entry in cpuinfo"
    @classmethod
    def from_stanza(cls, stanza):
        cpu_data = cls()
        cpu_data.update(dict([fact_sluggify_item(item) for item in stanza]))
        return cpu_data


class X86_64CpuinfoModel(CpuinfoModel):
    """The model for all the cpuinfo data for all processors on the machine.

    ie, all the data in /proc/cpuinfo field as opposed to X86_64ProcessModel which
    is the info for 1 processor."""
    fields_class = X86_64Fields


class Ppc64CpuinfoModel(CpuinfoModel):
    fields_class = Ppc64Fields


class Aarch64CpuinfoModel(CpuinfoModel):
    fields_class = Aarch64Fields


def fact_sluggify(key):
    """Encodes an arbitrary string to something that can be used as a fact name.

    ie, 'model_name' instead of 'Model name'
    whitespace -> _
    lowercase
    utf8
    escape quotes

    In theory, any utf8 would work
    """
    # yeah, terrible...
    return key.lower().strip().replace(' ', '_').replace('.', '_')


def fact_sluggify_item(item_tuple):
    newkey = fact_sluggify(item_tuple[0])
    return (newkey, item_tuple[1])


def split_key_value_generator(file_contents, line_splitter):
    for line in file_contents.splitlines():
        parts = line_splitter(line)
        if parts:
            yield parts


def line_splitter(line):
    # cpu family    : 6
    # model name    : Intel(R) Core(TM) i5 CPU       M 560  @ 2.67GHz
    parts = line.split(':', 1)
    if parts[0]:
        parts = [part.strip() for part in parts]
        return parts
    return None


def accumulate_fields(fields_accum, fields):
    for field in fields:
        fields_accum.add(field)
    return fields_accum


def find_shared_key_value_pairs(all_fields, processors):
    # smashem, last one wins
    smashed = collections.defaultdict(set)

    # build a dict of fieldname -> list of all the different values
    # so we can dump the variant ones.
    for field in all_fields:
        for k, v in [(field, processor.get(field)) for processor in processors]:
            if v is None:
                continue
            smashed[k].add(v)

    # remove fields that can't be smashed to one value
    common_cpu_info = dict([(x, smashed[x].pop()) for x in smashed if len(smashed[x]) == 1])
    return common_cpu_info


def split_kv_list_by_field(kv_list, field):
    """Split the iterable kv_list into chunks by field.

    For a list with repeating stanzas in it, this will
    return a generate that will return each chunk.

    For something like /proc/cpuinfo, called with
    field 'processor', each stanza is a different cpu.
    """
    current_stanza = None
    for key, value in kv_list:
        if key == field:
            if current_stanza:
                yield current_stanza
            current_stanza = [(key, value)]
            continue

        # if we have garbage in and no start to processor info
        if current_stanza:
            current_stanza.append((key, value))

    # end of kv_list
    if current_stanza:
        yield current_stanza


"""
Processor   : AArch64 Processor rev 0 (aarch64)
processor   : 0
processor   : 1
processor   : 2
processor   : 3
processor   : 4
processor   : 5
processor   : 6
processor   : 7
Features    : fp asimd evtstrm
CPU implementer : 0x50
CPU architecture: AArch64
CPU variant : 0x0
CPU part    : 0x000
CPU revision    : 0

Hardware    : APM X-Gene Mustang board
"""


class BaseCpuInfo(object):
    @classmethod
    def from_proc_cpuinfo_string(cls, proc_cpuinfo_string):
        """Return a BaseCpuInfo subclass based on proc_cpuinfo_string.

        proc_cpuinfo_string is the string resulting from reading
        the entire contents of /proc/cpuinfo."""
        cpu_info = cls()
        cpu_info._parse(proc_cpuinfo_string)

        return cpu_info


class Aarch64CpuInfo(BaseCpuInfo):
    def __init__(self):
        self.cpu_info = Aarch64CpuinfoModel()

    def _parse(self, cpuinfo_data):
        raw_kv_iter = split_key_value_generator(cpuinfo_data, line_splitter)

        # Yes, there is a 'Processor' field and multiple lower case 'processor'
        # fields.
        kv_iter = (self._capital_processor_to_model_name(item)
                   for item in raw_kv_iter)

        slugged_kv_list = [fact_sluggify_item(item) for item in kv_iter]

        # kind of duplicated
        self.cpu_info.common = self.gather_cpu_info_model(slugged_kv_list)
        self.cpu_info.processors = self.gather_processor_list(slugged_kv_list)

        # For now, 'hardware' is per
        self.cpu_info.other = self.gather_cpu_info_other(slugged_kv_list)

    def _capital_processor_to_model_name(self, item):
        """Use the uppercase Processor field value as the model name.

        For aarch64, the 'Processor' field is the closest to model name,
        so we sub it in now."""
        if item[0] == 'Processor':
            item[0] = "model_name"
        return item

    def gather_processor_list(self, kv_list):
        processor_list = []
        for k, v in kv_list:
            if k != 'processor':
                continue
            # build a ProcessorModel subclass for each processor
            # to add to CpuInfoModel.processors list
            cpu_info_model = self.gather_cpu_info_model(kv_list)
            cpu_info_model['processor'] = v
            processor_list.append(cpu_info_model)
        return processor_list

    # FIXME: more generic would be to split the stanzas by empty lines in the
    # first pass
    def gather_cpu_info_other(self, kv_list):
        other_list = []
        for k, v in kv_list:
            if k == 'hardware':
                other_list.append([k, v])
        return other_list

    def gather_cpu_info_model(self, kv_list):
        cpu_data = Aarch64ProcessorModel()
        for k, v in kv_list:
            if k == 'processor' or k == 'hardware':
                continue
            cpu_data[k] = v
        return cpu_data


class X86_64CpuInfo(BaseCpuInfo):
    def __init__(self):
        self.cpu_info = X86_64CpuinfoModel()

    def _parse(self, cpuinfo_data):
        # ordered list
        kv_iter = split_key_value_generator(cpuinfo_data, line_splitter)

        processors = []
        all_fields = set()
        for processor_stanza in split_kv_list_by_field(kv_iter, 'processor'):
            proc_dict = self.processor_stanza_to_processor_data(processor_stanza)
            processors.append(proc_dict)
            # keep track of fields as we see them
            all_fields = accumulate_fields(all_fields, list(proc_dict.keys()))

        self.cpu_info.common = find_shared_key_value_pairs(all_fields, processors)
        self.cpu_info.processors = processors
        self.cpu_info.cpuinfo_data = cpuinfo_data

    def processor_stanza_to_processor_data(self, stanza):
        "Take a list of k,v tuples, sluggify name, and add to a dict."
        cpu_data = X86_64ProcessorModel()
        cpu_data.update(dict([fact_sluggify_item(item) for item in stanza]))
        return cpu_data


class Ppc64CpuInfo(BaseCpuInfo):
    def __init__(self):
        self.cpu_info = Ppc64CpuinfoModel()

    def _parse(self, cpuinfo_data):
        kv_iter = split_key_value_generator(cpuinfo_data, line_splitter)

        processor_iter = itertools.takewhile(self._not_timebase_key, kv_iter)
        for processor_stanza in split_kv_list_by_field(processor_iter, 'processor'):
            proc_dict = Ppc64ProcessorModel.from_stanza(processor_stanza)
            self.cpu_info.processors.append(proc_dict)

        # Treat the rest of the info as shared between all of the processor entries
        # kv_iter is the rest of cpuinfo that isn't processor stanzas
        self.cpu_info.common = dict([fact_sluggify_item(item) for item in kv_iter])
        self.cpu_info.cpuinfo_data = cpuinfo_data

    def _not_timebase_key(self, item):
        return item[0] != 'timebase'


class SystemCpuInfoFactory(object):
    uname_to_cpuinfo = {'x86_64': X86_64CpuInfo,
                        'aarch64': Aarch64CpuInfo,
                        'ppc64': Ppc64CpuInfo,
                        'ppc64le': Ppc64CpuInfo}
    proc_cpuinfo_path = '/proc/cpuinfo'

    @classmethod
    def from_uname_machine(cls, uname_machine, prefix=None):
        if uname_machine not in SystemCpuInfoFactory.uname_to_cpuinfo:
            # er?
            raise NotImplementedError

        proc_cpuinfo_string = cls.open_proc_cpuinfo(prefix)

        arch_class = cls.uname_to_cpuinfo[uname_machine]
        return arch_class.from_proc_cpuinfo_string(proc_cpuinfo_string)

    @classmethod
    def open_proc_cpuinfo(cls, prefix=None):
        proc_cpuinfo_path = cls.proc_cpuinfo_path
        if prefix:
            proc_cpuinfo_path = os.path.join(prefix, cls.proc_cpuinfo_path[1:])
        proc_cpuinfo_buf = ''
        with open(proc_cpuinfo_path, 'r') as proc_cpuinfo_f:
            proc_cpuinfo_buf = proc_cpuinfo_f.read()
        return proc_cpuinfo_buf



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2014 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import logging

from rhsm.certificate2 import CONTENT_ACCESS_CERT_TYPE

log = logging.getLogger(__name__)

# These containerish iterables could share a
# base class, though, it should probably just
# be based on containers.abc.Iterable


class Content(object):
    """
    A generic representation of entitled content.
    """
    def __init__(self, content_type, name, label,
                 url=None, gpg=None, tags=None, cert=None,
                 enabled=None, metadata_expire=None):
        self.content_type = content_type
        self.name = name
        self.label = label

        self.url = url
        self.gpg = gpg
        self.tags = tags or []
        self.cert = cert
        self.enabled = enabled
        self.metadata_expire = metadata_expire


class Entitlement(object):
    """Represent an entitlement.

    Has a 'contents' attribute that is an
    iterable of EntitledContent objects. (currently EntCertEntitledContent)

    Note 'contents' that differs from the 'content'naming the
    rhsm EntitlementCertificate object uses.
    """

    def __init__(self, contents=None, entitlement_type=None):
        self.contents = contents
        self.entitlement_type = entitlement_type


class EntitlementSource(object):
    """Populate with info needed for plugins to find content.

    Acts as a iterable over entitlements.
    """
    def __init__(self):
        self._entitlements = []
        self.product_tags = []

    def __iter__(self):
        return iter(self._entitlements)

    def __len__(self):
        return len(self._entitlements)

    def __getitem__(self, key):
        return self._entitlements[key]


def find_content(ent_source, content_type=None):
    """
    Scan all entitlements looking for content of the given type. (string)
    Type will be compared case insensitive.

    Returns a list of model.Content.
    """
    entitled_content = []
    content_access_entitlement_content = {}
    content_labels = set()
    log.debug("Searching for content of type: %s" % content_type)
    for entitlement in ent_source:
        for content in entitlement.contents:
            # this is basically matching_content from repolib
            if content.content_type.lower() == content_type.lower() and \
                    content_tag_match(content.tags, ent_source.product_tags):
                if entitlement.entitlement_type == CONTENT_ACCESS_CERT_TYPE:
                    content_access_entitlement_content[content.label] = content
                else:
                    entitled_content.append(content)
                    content_labels.add(content.label)

    # now add content that wasn't covered by basic entitlement certs
    for label, content in list(content_access_entitlement_content.items()):
        if label not in content_labels:
            entitled_content.append(content)
    return entitled_content


def content_tag_match(content_tags, product_tags):
    """See if content required tags are provided by installed products.

    Note: this is skipped if the content does not have any required tags.
    """

    all_tags_found = True
    for content_tag in content_tags:
        if content_tag not in product_tags:
            all_tags_found = False
    return all_tags_found



from __future__ import print_function, division, absolute_import

from rhsmlib.dbus.facts.base import HostFacts  # noqa: F401
from rhsmlib.dbus.facts.client import FactsClient, FactsClientAuthenticationError  # noqa: F401
from rhsmlib.dbus.facts.constants import FACTS_DBUS_NAME, FACTS_DBUS_INTERFACE, FACTS_DBUS_PATH  # noqa: F401



from __future__ import print_function, division, absolute_import

from gtk.gdk import Pixbuf as _Pixbuf
from gtk.gdk import pixbuf_new_from_file_at_size


class Pixbuf(_Pixbuf):
    # FIXME: would monkey patching this onto _Pixbuf be better?
    @classmethod
    def new_from_file_at_size(cls, path, height, width):
        "Return a gtk.gdk.Pixbuf (note, not a ga.gtk.GdkPixbuf)"
        return pixbuf_new_from_file_at_size(path, height, width)


__all__ = [Pixbuf]



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2014 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

""" Core code for the container content plugin. """
import logging
import os
import re
import shutil

from subscription_manager import certlib
from subscription_manager.model import find_content

from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)

CONTAINER_CONTENT_TYPE = "containerimage"

RH_CDN_REGEX = re.compile('^cdn\.(?:.*\.)?redhat\.com$')
RH_CDN_CA = "/etc/rhsm/ca/redhat-entitlement-authority.pem"


class ContainerContentUpdateActionCommand(object):
    """
    UpdateActionCommand for Docker configuration.

    Return a ContainerContentUpdateReport.
    """
    def __init__(self, ent_source, registry_hostnames, host_cert_dir):
        self.ent_source = ent_source
        self.registry_hostnames = registry_hostnames
        self.host_cert_dir = host_cert_dir

    def perform(self):

        report = ContainerUpdateReport()

        content_sets = self._find_content()
        log.debug("Got content_sets: %s" % content_sets)
        unique_cert_paths = self._get_unique_paths(content_sets)

        for registry_hostname in self.registry_hostnames:
            cert_dir = ContainerCertDir(report=report,
                registry=registry_hostname,
                host_cert_dir=self.host_cert_dir)
            cert_dir.sync(unique_cert_paths)

        return report

    def _find_content(self):
        return find_content(self.ent_source,
            content_type=CONTAINER_CONTENT_TYPE)

    def _get_unique_paths(self, content_sets):
        """
        Return a list of unique keypairs to be copied into the
        docker certificates directory.
        """
        # Identify all the unique certificates we need to copy:
        unique_cert_paths = set()
        for content in content_sets:
            unique_cert_paths.add(
                KeyPair(content.cert.path, content.cert.key_path()))
        return unique_cert_paths


class KeyPair(object):
    """ Simple object to hold paths to an entitlement cert and key. """
    def __init__(self, cert_path, key_path):

        self.cert_path = cert_path
        self.key_path = key_path

        # Calculate the expected filenames for docker certs, just
        # re-use the base filename from entitlement cert and change
        # the file extension.
        self.dest_cert_filename = "%s.cert" %  \
            os.path.splitext(os.path.basename(self.cert_path))[0]
        # We use the base filename from the cert so avoid the SERIAL-key
        # we would have with our normal entitlement certs. We need the
        # base filename to be the same for both .cert and .key.
        self.dest_key_filename = "%s.key" % \
            os.path.splitext(os.path.basename(self.cert_path))[0]

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and
            self.cert_path == other.cert_path and
            self.key_path == other.key_path)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return "KeyPair<%s, %s>" % (self.cert_path, self.key_path)

    def __hash__(self):
        return hash(self.__repr__())


class ContainerCertDir(object):
    """
    An object to manage the docker certificate directory at
    /etc/docker/certs.d/.
    """

    # We will presume to manage files with these extensions in the
    # hostname directory we're dealing with. Any unexpected files with
    # these extensions will be removed. Any other files will be left
    # alone.
    MANAGED_EXTENSIONS = [".cert", ".key"]

    def __init__(self, report, registry, host_cert_dir):
        self.report = report
        self.registry = registry

        # This is the overall cert directory where hostname specific
        # subdirectories will go:
        self.host_cert_dir = host_cert_dir

        # The final hostname specific cert directory we'll sync to:
        self.path = os.path.join(self.host_cert_dir, registry)

    def sync(self, expected_keypairs):
        log.debug("Syncing container certificates to %s" % self.path)
        if not os.path.exists(self.host_cert_dir):
            log.warn("Container cert directory does not exist: %s" % self.host_cert_dir)
            log.warn("Exiting plugin")
            return
        if not os.path.exists(self.path):
            log.info("Container cert directory does not exist, creating it.")
            os.mkdir(self.path)

        # Build up the list of certificates that should be in the
        # directory. We'll use this later to prune out any that need to
        # be cleaned up.
        expected_files = []

        for keypair in expected_keypairs:
            full_cert_path = os.path.join(self.path,
                keypair.dest_cert_filename)
            full_key_path = os.path.join(self.path,
                keypair.dest_key_filename)
            expected_files.append(keypair.dest_cert_filename)
            expected_files.append(keypair.dest_key_filename)
            # WARNING: We assume that matching filenames must be the same
            # certificate and key. Because we use serials in the filename, this
            # should be safe.
            if not os.path.exists(full_cert_path):
                log.info("Copying: %s -> %s" %
                    (keypair.cert_path, full_cert_path))
                shutil.copyfile(keypair.cert_path, full_cert_path)
                shutil.copymode(keypair.cert_path, full_cert_path)
                self.report.added.append(full_cert_path)
            if not os.path.exists(full_key_path):
                log.info("Copying: %s -> %s" %
                    (keypair.key_path, full_key_path))
                shutil.copyfile(keypair.key_path, full_key_path)
                shutil.copymode(keypair.key_path, full_key_path)
                self.report.added.append(full_key_path)

        self._prune_old_certs(expected_files)

        # If we see something that looks like Red Hat's CDN, we know we need
        # to symlink the python-rhsm delivered CA cert in:
        if RH_CDN_REGEX.match(os.path.basename(self.path)):
            if not self._rh_cdn_ca_exists():
                log.error("Detected a CDN hostname, but no CA certificate installed.")
            else:
                outfile = "%s.crt" % os.path.splitext(os.path.basename(RH_CDN_CA))[0]
                ca_symlink = os.path.join(self.path, outfile)
                if not os.path.exists(ca_symlink):
                    os.symlink(RH_CDN_CA, ca_symlink)
                    log.info("Created symlink: %s -> %s" % (ca_symlink, RH_CDN_CA))

    def _rh_cdn_ca_exists(self):
        """
        Check if the python-rhsm delivered CA PEM exists.
        """
        # Separate method for testing purposes.
        return os.path.exists(RH_CDN_CA)

    def _prune_old_certs(self, expected_files):
        """
        Returns the base filenames of each file in the destination directory.
        """
        for f in os.listdir(self.path):
            fullpath = os.path.join(self.path, f)
            if os.path.isfile(fullpath) and \
                os.path.splitext(f)[1] in self.MANAGED_EXTENSIONS and \
                not f in expected_files:
                    log.info("Cleaning up old certificate: %s" % f)
                    os.remove(fullpath)
                    self.report.removed.append(fullpath)


class ContainerUpdateReport(certlib.ActionReport):
    """Track container cert changes."""
    name = "Container certificate updates report"

    def __init__(self):
        super(ContainerUpdateReport, self).__init__()
        # Full path to certs/keys we added:
        self.added = []

        # Full path to cert/keys we cleaned up:
        self.removed = []

    def updates(self):
        """ Number of updates. """
        return len(self.added) + len(self.removed)

    def _format_file_list(self, file_list):
        s = []
        for filename in file_list:
            s.append(file_list)
        return '\n'.join(s)

    def __str__(self):
        s = ["Container content cert updates\n"]
        s.append(_("Added:"))
        s.append(self._format_file_list(self.added))
        s.append(_("Removed:"))
        s.append(self._format_file_list(self.removed))
        return '\n'.join(s)



from __future__ import print_function, division, absolute_import

import tempfile
import shutil
from mock import patch

from . import fixture
from subscription_manager import facts
from rhsm import ourjson as json

facts_buf = """
{
    "another": "blargh",
    "caneatcheese": "automatic100",
    "cpu.core(s)_per_socket": 4,
    "cpu.cpu(s)": 8,
    "cpu.cpu_mhz": "2420",
    "cpu.cpu_socket(s)": 2,
    "distribution.id": "Santiago",
    "distribution.name": "Red Hat Enterprise Linux Server",
    "distribution.version": "6.0",
    "dmi.baseboard.manufacturer": "Dell Inc.",
    "dmi.baseboard.product_name": "0RW203",
    "dmi.baseboard.serial_number": "..CN1374094G001M.",
    "dmi.baseboard.version": "",
    "dmi.bios.address": "0xf0000",
    "dmi.bios.bios_revision": "0.0",
    "dmi.bios.relase_date": "08/21/2008",
    "dmi.bios.rom_size": "1024 KB",
    "dmi.bios.runtime_size": "64 KB",
    "dmi.bios.vendor": "Dell Inc.",
    "dmi.bios.version": "A04",
    "dmi.chassis.asset_tag": "",
    "dmi.chassis.boot-up_state": "Warning",
    "dmi.chassis.lock": "Not Present",
    "dmi.chassis.manufacturer": "Dell Inc.",
    "dmi.chassis.power_supply_state": "Safe",
    "dmi.chassis.security_status": "None",
    "dmi.chassis.serial_number": "B654BK1",
    "dmi.chassis.thermal_state": "Safe",
    "dmi.chassis.type": "Tower",
    "dmi.chassis.version": "Not Specified",
    "dmi.connector.external_connector_type": "Mini Jack (headphones)",
    "dmi.connector.external_reference_designator": "Not Specified",
    "dmi.connector.internal_connector_type": "None",
    "dmi.connector.internal_reference_designator": "LINE-IN",
    "dmi.connector.port_type": "Audio Port",
    "dmi.memory.array_handle": "0x1000",
    "dmi.memory.assettag": "FFFFFF",
    "dmi.memory.bank_locator": "Not Specified",
    "dmi.memory.data_width": "64 bit",
    "dmi.memory.error_correction_type": "Multi-bit ECC",
    "dmi.memory.error_information_handle": "No Error",
    "dmi.memory.form_factor": "FB-DIMM",
    "dmi.memory.location": "System Board Or Motherboard",
    "dmi.memory.locator": "DIMM 4",
    "dmi.memory.manufacturer": "FFFFFFFFFFFF",
    "dmi.memory.maximum_capacity": "64 GB",
    "dmi.memory.part_number": "",
    "dmi.memory.serial_number": "FFFFFFFF",
    "dmi.memory.size": "4096 MB",
    "dmi.memory.speed": "667 MHz (1.5ns)",
    "dmi.memory.total_width": "72 bit",
    "dmi.memory.type": "DDR2 FB-DIMM",
    "dmi.memory.use": "System Memory",
    "dmi.processor.asset_tag": "Not Specified",
    "dmi.processor.family": "Xeon",
    "dmi.processor.l1_cache_handle": "0x0702",
    "dmi.processor.l2_cache_handle": "0x0703",
    "dmi.processor.part_number": "Not Specified",
    "dmi.processor.serial_number": "Not Specified",
    "dmi.processor.socket_designation": "CPU",
    "dmi.processor.status": "Populated:Idle",
    "dmi.processor.type": "Central Processor",
    "dmi.processor.upgrade": "Socket LGA771",
    "dmi.processor.version": "Not Specified",
    "dmi.processor.voltage": "1.1 V",
    "dmi.slot.current_usage": "Available",
    "dmi.slot.designation": "SLOT1",
    "dmi.slot.slotid": "1",
    "dmi.slot.slotlength": "Long",
    "dmi.slot.type:slotbuswidth": "x4",
    "dmi.slot.type:slottype": "PCI Express",
    "dmi.system.family": "Not Specified",
    "dmi.system.manufacturer": "Dell Inc.",
    "dmi.system.product_name": "Precision WorkStation T5400",
    "dmi.system.serial_number": "B654BK1",
    "dmi.system.sku_number": "Not Specified",
    "dmi.system.status": "No errors detected",
    "dmi.system.uuid": "44454c4c-3600-1035-8034-c2c04f424b31",
    "dmi.system.version": "Not Specified",
    "dmi.system.wake-up_type": "Power Switch",
    "lscpu.architecture": "x86_64",
    "lscpu.cpu_family": "6",
    "lscpu.cpu_mhz": "1995.095",
    "lscpu.cpu_op-mode(s)": "32-bit, 64-bit",
    "lscpu.cpu_socket(s)": "2",
    "lscpu.l1d_cache": "32K",
    "lscpu.l1i_cache": "32K",
    "lscpu.l2_cache": "6144K",
    "lscpu.model": "23",
    "lscpu.numa_node(s)": "1",
    "lscpu.numa_node0_cpu(s)": "0-7",
    "lscpu.stepping": "10",
    "lscpu.thread(s)_per_core": "1",
    "lscpu.vendor_id": "GenuineIntel",
    "lscpu.virtualization": "VT-x",
    "memory.memtotal": "10326220",
    "memory.swaptotal": "12419068",
    "nxxxw.tddng3": "10d0",
    "system.certificate_version": "3.2",
    "system.default_locale": "en_US",
    "test.attr": "blippy2",
    "uname.machine": "x86_64",
    "uname.release": "2.6.35.11-83.fc14.x86_64",
    "uname.sysname": "Linux",
    "uname.version": "#1 SMP Mon Feb 7 07:06:44 UTC 2011",
    "virt.host_type": "",
    "virt.is_guest": false
}

"""


def stub_get_facts():
    return {'newstuff': True}


class TestFacts(fixture.SubManFixture):
    def setUp(self):
        super(TestFacts, self).setUp()

        self.fact_cache_dir = tempfile.mkdtemp()
        fact_cache = self.fact_cache_dir + "/facts.json"
        fd = open(fact_cache, "w")
        fd.write(facts_buf)
        fd.close()
        self.f = facts.Facts()
        self.f.CACHE_FILE = fact_cache

    def tearDown(self):
        super(TestFacts, self).tearDown()
        shutil.rmtree(self.fact_cache_dir)

    def test_facts_read(self):
        cached_facts = self.f._read_cache()
        self.assertEqual(cached_facts["test.attr"], "blippy2")

    def test_facts_last_update(self):
        #FIXME: verify the date is correct
        self.f.get_last_update()

    @patch('subscription_manager.facts.Facts.get_facts',
           return_value={'newstuff': 'a new_hope'})
    def test_facts_has_changed(self, mock_collect):
        self.assertTrue(self.f.has_changed())

    @patch('subscription_manager.facts.Facts.get_facts')
    def test_facts_has_changed_no_change(self, mock_collect):
        test_facts = json.loads(facts_buf)
        mock_collect.return_value = test_facts
        changed = self.f.has_changed()
        self.assert_equal_dict(test_facts, self.f.facts)
        self.assertFalse(changed)

    @patch('subscription_manager.facts.Facts.get_facts')
    def test_facts_has_changed_with_change(self, mock_collect):
        test_facts = json.loads(facts_buf)
        # change socket fact count from what is in the cache
        test_facts['cpu.cpu_socket(s)'] = '16'
        mock_collect.return_value = test_facts

        changed = self.f.has_changed()
        self.assertEqual(self.f.facts['cpu.cpu_socket(s)'], '16')
        self.assertTrue(changed)

    @patch('subscription_manager.facts.Facts._read_cache',
           return_value=None)
    @patch('subscription_manager.facts.Facts.get_facts')
    def test_facts_has_changed_cache_is_none(self, mock_collect, mock_read_cache):
        test_facts = json.loads(facts_buf)
        mock_collect.return_value = test_facts

        changed = self.f.has_changed()
        self.assert_equal_dict(test_facts, self.f.facts)
        self.assertTrue(changed)

    @patch('subscription_manager.facts.Facts._cache_exists',
           return_value=False)
    @patch('subscription_manager.facts.Facts.get_facts')
    def test_facts_has_changed_cache_exists_false(self, mock_collect, mock_read_cache):
        test_facts = json.loads(facts_buf)
        mock_collect.return_value = test_facts

        changed = self.f.has_changed()
        self.assertTrue(changed)

    @patch('subscription_manager.facts.Facts.get_facts')
    def test_get_facts(self, mock_collect):
        mock_collect.return_value = \
            {'net.interface.lo.ipv4_address': '127.0.0.1'}

        f = self.f.get_facts()

        self.assertTrue(isinstance(f, dict))
        self.assertEqual(f['net.interface.lo.ipv4_address'], '127.0.0.1')



from __future__ import print_function, division, absolute_import

import mock

from . import fixture

from . import test_model

from rhsm import certificate2
from subscription_manager import model
from subscription_manager.model import ent_cert
from subscription_manager import injection as inj


class TestEntitlement(fixture.SubManFixture):
    def test_empty_init(self):
        e = model.Entitlement()
        self.assertTrue(hasattr(e, 'contents'))

    def test_init_empty_contents(self):
        e = model.Entitlement(contents=[])
        self.assertTrue(hasattr(e, 'contents'))
        self.assertEqual(e.contents, [])

    def test_contents(self):
        contents = [mock.Mock(), mock.Mock()]
        e = model.Entitlement(contents=contents)
        self.assertTrue(hasattr(e, 'contents'))
        self.assertEqual(len(e.contents), 2)

        for a_content in e.contents:
            self.assertTrue(isinstance(a_content, mock.Mock))

        self.assertTrue(isinstance(e.contents[0], mock.Mock))


def create_mock_content(name=None, url=None, gpg=None, enabled=None, content_type=None, tags=None):
    mock_content = mock.Mock()
    mock_content.name = name or "mock_content"
    mock_content.url = url or "http://mock.example.com"
    mock_content.gpg = gpg or "path/to/gpg"
    mock_content.enabled = enabled or True
    mock_content.content_type = content_type or "yum"
    mock_content.tags = tags or []
    return mock_content


class TestEntitlementCertContent(test_model.TestContent):
    def test_from_cert_content_yum(self):
        mock_content = create_mock_content()

        ent_cert_content = ent_cert.EntitlementCertContent.from_cert_content(mock_content)
        self._check_attrs(ent_cert_content)

    def test_from_cert_content_ostree(self):
        mock_content = create_mock_content(content_type="ostree")

        ent_cert_content = ent_cert.EntitlementCertContent.from_cert_content(mock_content)
        self._check_attrs(ent_cert_content)

    def test_from_cert_content_ostree_tags(self):
        mock_content = create_mock_content(content_type="ostree", tags=["rhel-7-atomic"])

        ent_cert_content = ent_cert.EntitlementCertContent.from_cert_content(mock_content)
        self._check_attrs(ent_cert_content)


class TestEntitlementCertEntitlement(TestEntitlement):
    def test_from_ent_cert(self):
        mock_content = create_mock_content()

        contents = [mock_content]

        mock_ent_cert = mock.Mock()
        mock_ent_cert.content = contents

        ece = model.ent_cert.EntitlementCertEntitlement.from_ent_cert(mock_ent_cert)

        self.assertEqual(ece.contents[0].name, contents[0].name)
        self.assertEqual(ece.contents[0].label, contents[0].label)
        self.assertEqual(ece.contents[0].gpg, contents[0].gpg)
        self.assertEqual(ece.contents[0].content_type,
            contents[0].content_type)
        self.assertEqual(len(ece.contents), 1)

        # for ostree content, gpg is likely to change
        self.assertEqual(ece.contents[0].gpg, mock_content.gpg)


# FIXME: move to stubs/fixture, copied from ent_branding
# The installed product ids don't have brand_type/brand_name, just the
# Product from the ent cert
class DefaultStubInstalledProduct(certificate2.Product):
    def __init__(self, id=123, name="Awesome OS",
                 provided_tags=None,
                 brand_type=None, brand_name=None):

        tags = provided_tags or ["awesomeos-ostree-1"]
        super(DefaultStubInstalledProduct, self).__init__(id=id, name=name, provided_tags=tags,
                                                          brand_type=brand_type,
                                                          brand_name=brand_name)


class TestEntitlementDirEntitlementSource(test_model.TestEntitlementSource):
    def setUp(self):
        super(TestEntitlementDirEntitlementSource, self).setUp()
        self._inj_mock_dirs()

    def _inj_mock_dirs(self, stub_product=None):

        stub_product = stub_product or DefaultStubInstalledProduct()
        mock_prod_dir = mock.NonCallableMock(name='MockProductDir')
        mock_prod_dir.get_installed_products.return_value = [stub_product.id]
        mock_prod_dir.get_provided_tags.return_value = stub_product.provided_tags

        mock_content = create_mock_content(tags=['awesomeos-ostree-1'])
        mock_cert_contents = [mock_content]

        mock_ent_cert = mock.Mock(name='MockEntCert')
        mock_ent_cert.products = [stub_product]
        mock_ent_cert.content = mock_cert_contents

        mock_ent_dir = mock.NonCallableMock(name='MockEntDir')
        mock_ent_dir.list_valid.return_value = [mock_ent_cert]
        mock_ent_dir.list_valid_with_content_access.return_value = [mock_ent_cert]

        inj.provide(inj.PROD_DIR, mock_prod_dir)
        inj.provide(inj.ENT_DIR, mock_ent_dir)

    def _stub_product(self):
        return DefaultStubInstalledProduct

    def test_init_no_matching(self):
        # add a product that will not match product tags
        self._inj_mock_dirs(stub_product=DefaultStubInstalledProduct(provided_tags=[]))
        ecc = ent_cert.EntitlementDirEntitlementSource()
        self.assertEqual(len(ecc), 1)
        self.assertEqual(len(ecc.product_tags), 1)
